@@ -1,7 +1,6 @@
 /*
-    $Id: util.h,v 1.12 2008/03/25 15:59:10 karl Exp $
-
-    Copyright (C) 2004, 2005, 2006, 2008 Rocky Bernstein <rocky@gnu.org>
+    Copyright (C) 2004, 2005, 2006, 2008, 2010, 2012
+    Rocky Bernstein <rocky@gnu.org>
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
     This program is free software: you can redistribute it and/or modify
@@ -18,8 +17,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef __CDIO_UTIL_H__
-#define __CDIO_UTIL_H__
+#ifndef CDIO_UTIL_H_
+#define CDIO_UTIL_H_
 
 /*!
    \file util.h 
@@ -28,6 +27,19 @@
    Warning: this will probably get removed/replaced by using glib.h
 */
 #include <stdlib.h>
+#include <cdio/types.h>
+
+#if !defined CDIO_INLINE
+#if defined(__cplusplus) || defined(inline)
+#define CDIO_INLINE inline
+#elif defined(__GNUC__)
+#define CDIO_INLINE __inline__
+#elif defined(_MSC_VER)
+#define CDIO_INLINE __inline
+#else
+#define CDIO_INLINE
+#endif
+#endif /* CDIO_INLINE */
 
 #undef  MAX
 #define MAX(a, b)  (((a) > (b)) ? (a) : (b))
@@ -41,7 +53,7 @@
 #undef  CLAMP
 #define CLAMP(x, low, high)  (((x) > (high)) ? (high) : (((x) < (low)) ? (low) : (x)))
 
-static inline uint32_t
+static CDIO_INLINE uint32_t
 _cdio_len2blocks (uint32_t i_len, uint16_t i_blocksize)
 {
   uint32_t i_blocks;
@@ -54,13 +66,13 @@ _cdio_len2blocks (uint32_t i_len, uint16_t i_blocksize)
 }
 
 /* round up to next block boundary */
-static inline unsigned 
+static CDIO_INLINE unsigned 
 _cdio_ceil2block (unsigned offset, uint16_t i_blocksize)
 {
   return _cdio_len2blocks (offset, i_blocksize) * i_blocksize;
 }
 
-static inline unsigned int
+static CDIO_INLINE unsigned int
 _cdio_ofs_add (unsigned offset, unsigned length, uint16_t i_blocksize)
 {
   if (i_blocksize - (offset % i_blocksize) < length)
@@ -71,7 +83,7 @@ _cdio_ofs_add (unsigned offset, unsigned length, uint16_t i_blocksize)
   return offset;
 }
 
-static inline const char *
+static CDIO_INLINE const char *
 _cdio_bool_str (bool b)
 {
   return b ? "yes" : "no";
@@ -87,6 +99,12 @@ _cdio_memdup (const void *mem, size_t count);
 char *
 _cdio_strdup_upper (const char str[]);
 
+/* Duplicate path and make it platform compliant. Typically needed for
+   MinGW/MSYS where a "/c/..." path must be translated to "c:/..." for
+   use with fopen(), etc. Returned string must be freed by the caller. */
+char * 
+_cdio_strdup_fixpath (const char path[]);
+
 void
 _cdio_strfreev(char **strv);
 
@@ -99,13 +117,19 @@ _cdio_strsplit(const char str[], char delim);
 uint8_t cdio_to_bcd8(uint8_t n);
 uint8_t cdio_from_bcd8(uint8_t p);
 
-void cdio_follow_symlink (const char * src, char * dst);
+/*!  cdio_realpath() same as POSIX.1-2001 realpath if that's
+around. If not we do poor-man's simulation of that behavior.  */
+char *cdio_realpath (const char *psz_src, char *psz_dst);
+
+#ifdef WANT_FOLLOW_SYMLINK_COMPATIBILITY
+# define cdio_follow_symlink cdio_realpath
+#endif
   
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* __CDIO_UTIL_H__ */
+#endif /* CDIO_UTIL_H_ */
 
 
 /* 
