@@ -1,21 +1,19 @@
 /*
-    $Id: win32.c,v 1.33 2006/03/17 03:10:53 rocky Exp $
+  Copyright (C) 2003, 2004, 2005, 2006, 2008, 2010, 2011, 2012
+  Rocky Bernstein <rocky@gnu.org>
 
-    Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
+  This program is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* This file contains Win32-specific code and implements low-level 
@@ -26,12 +24,15 @@
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: win32.c,v 1.33 2006/03/17 03:10:53 rocky Exp $";
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif 
 
 #include <cdio/cdio.h>
 #include <cdio/sector.h>
 #include <cdio/util.h>
 #include <cdio/mmc.h>
+#include <cdio/logging.h>
 #include "cdio_assert.h"
 #include "cdio_private.h" /* protoype for cdio_is_device_win32 */
 
@@ -70,7 +71,7 @@ static const char _rcsid[] = "$Id: win32.c,v 1.33 2006/03/17 03:10:53 rocky Exp
 #include <sys/types.h>
 #endif
 
-#if defined (MSVC) || defined (_XBOX)
+#if defined (_MSC_VER) || defined (_XBOX)
 #undef IN
 #else
 #include "aspi32.h"
@@ -199,7 +200,7 @@ _cdio_mciSendCommand(int id, UINT msg, DWORD flags, void *arg)
 #else
   MCIERROR mci_error;
   
-  mci_error = mciSendCommand(id, msg, flags, (DWORD)arg);
+  mci_error = mciSendCommand(id, msg, flags, (DWORD_PTR)arg);
   if ( mci_error ) {
     char error[256];
     
@@ -222,12 +223,16 @@ str_to_access_mode_win32(const char *psz_access_mode)
     return _AM_IOCTL;
   else if (!strcmp(psz_access_mode, "ASPI")) {
 #ifdef _XBOX
-    return _AM_ASPI;
-#else 
     cdio_warn ("XBOX doesn't support access type: %s. Default used instead.", 
 	       psz_access_mode);
     return default_access_mode;
+#else 
+    return _AM_ASPI;
 #endif    
+  } else if (!strcmp(psz_access_mode, "MMC_RDWR")) {
+    return _AM_MMC_RDWR;
+  } else if (!strcmp(psz_access_mode, "MMC_RDWR_EXCL")) {
+    return _AM_MMC_RDWR_EXCL;
   } else {
     cdio_warn ("unknown access type: %s. Default used instead.", 
 	       psz_access_mode);
@@ -303,7 +308,6 @@ init_win32 (void *p_user_data)
   
   p_env->gen.init           = true;
   p_env->gen.toc_init       = false;
-  p_env->gen.b_cdtext_init  = false;
   p_env->gen.b_cdtext_error = false;
   p_env->gen.fd             = open (p_env->gen.source_name, O_RDONLY, 0);
 
@@ -316,10 +320,17 @@ init_win32 (void *p_user_data)
   p_env->b_ioctl_init    = false;
 
 
-  if ( _AM_IOCTL == p_env->access_mode ) {
+  switch (p_env->access_mode) {
+  case _AM_IOCTL:
+  case _AM_MMC_RDWR:
+  case _AM_MMC_RDWR_EXCL:
     b_ret = init_win32ioctl(p_env);
-  } else {
+    break;
+  case _AM_ASPI:
     b_ret = init_aspi(p_env);
+    break;
+  default:
+    return 0;
   }
 
   /* It looks like get_media_changed_mmc will always
@@ -347,7 +358,7 @@ free_win32 (void *p_user_data)
   if( p_env->h_device_handle )
     CloseHandle( p_env->h_device_handle );
   if( p_env->hASPI )
-    FreeLibrary( (HMODULE)p_env->hASPI );
+    FreeLibrary( p_env->hASPI );
 
   free (p_env);
 }
@@ -374,12 +385,30 @@ read_audio_sectors (void *p_user_data, void *p_buf, lsn_t i_lsn,
 }
 
 /*!
-   Reads a single mode1 sector from cd device into data starting
-   from lsn. Returns 0 if no error. 
+   Reads an audio device into data starting from lsn.
+   Returns 0 if no error. 
+ */
+static driver_return_code_t
+read_data_sectors_win32 (void *p_user_data, void *p_buf, lsn_t i_lsn, 
+			 uint16_t i_blocksize, uint32_t i_blocks) 
+{
+  driver_return_code_t rc =
+    read_data_sectors_mmc(p_user_data, p_buf, i_lsn, i_blocksize, i_blocks);
+  if ( DRIVER_OP_SUCCESS != rc  ) {
+    /* Try using generic "cooked" mode. */
+    return read_data_sectors_generic( p_user_data, p_buf, i_lsn, 
+				      i_blocksize, i_blocks );
+  } 
+  return DRIVER_OP_SUCCESS;
+}
+
+/*!
+   Reads a single mode1 sector from cd device into data starting from
+   lsn. Returns 0 if no error.
  */
 static int
-read_mode1_sector (void *p_user_data, void *p_buf, lsn_t lsn, 
-		   bool b_form2)
+read_mode1_sector_win32 (void *p_user_data, void *p_buf, lsn_t lsn, 
+			 bool b_form2)
 {
   _img_private_t *p_env = p_user_data;
 
@@ -410,8 +439,8 @@ read_mode1_sector (void *p_user_data, void *p_buf, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-read_mode1_sectors (void *p_user_data, void *p_buf, lsn_t lsn, 
-		    bool b_form2, unsigned int nblocks)
+read_mode1_sectors_win32 (void *p_user_data, void *p_buf, lsn_t lsn, 
+			  bool b_form2, unsigned int nblocks)
 {
   _img_private_t *p_env = p_user_data;
   int i;
@@ -419,13 +448,13 @@ read_mode1_sectors (void *p_user_data, void *p_buf, lsn_t lsn,
 
   for (i = 0; i < nblocks; i++) {
     if (b_form2) {
-      if ( (retval = read_mode1_sector (p_env, 
-					  ((char *)p_buf) + (M2RAW_SECTOR_SIZE * i),
-					  lsn + i, true)) )
-	return retval;
+      retval = read_mode1_sector_win32 (p_env, ((char *)p_buf) + 
+					(M2RAW_SECTOR_SIZE * i),
+					lsn + i, true);
+      if ( retval ) return retval;
     } else {
       char buf[M2RAW_SECTOR_SIZE] = { 0, };
-      if ( (retval = read_mode1_sector (p_env, buf, lsn + i, false)) )
+      if ( (retval = read_mode1_sector_win32 (p_env, buf, lsn + i, false)) )
 	return retval;
       
       memcpy (((char *)p_buf) + (CDIO_CD_FRAMESIZE * i), 
@@ -440,8 +469,8 @@ read_mode1_sectors (void *p_user_data, void *p_buf, lsn_t lsn,
    from lsn. Returns 0 if no error. 
  */
 static int
-read_mode2_sector (void *p_user_data, void *data, lsn_t lsn, 
-		    bool b_form2)
+read_mode2_sector_win32 (void *p_user_data, void *data, lsn_t lsn, 
+			 bool b_form2)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
   _img_private_t *p_env = p_user_data;
@@ -480,17 +509,17 @@ read_mode2_sector (void *p_user_data, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-read_mode2_sectors (void *p_user_data, void *data, lsn_t lsn, 
-		    bool b_form2, unsigned int i_blocks)
+read_mode2_sectors_win32 (void *p_user_data, void *data, lsn_t lsn, 
+			  bool b_form2, unsigned int i_blocks)
 {
   int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
   for (i = 0; i < i_blocks; i++) {
-    if ( (retval = read_mode2_sector (p_user_data, 
-				      ((char *)data) + (blocksize * i),
-				      lsn + i, b_form2)) )
+    if ( (retval = read_mode2_sector_win32 (p_user_data, 
+					    ((char *)data) + (blocksize * i),
+					    lsn + i, b_form2)) )
       return retval;
   }
   return 0;
@@ -569,7 +598,6 @@ open_close_media_win32 (const char *psz_win32_drive, DWORD command_flags)
   return DRIVER_OP_UNSUPPORTED;
 #else
   MCI_OPEN_PARMS op;
-  MCI_STATUS_PARMS st;
   DWORD i_flags;
   int ret;
     
@@ -582,7 +610,6 @@ open_close_media_win32 (const char *psz_win32_drive, DWORD command_flags)
     MCI_OPEN_ELEMENT | MCI_OPEN_SHAREABLE;
   
   if( _cdio_mciSendCommand( 0, MCI_OPEN, i_flags, &op ) ) {
-    st.dwItem = MCI_STATUS_READY;
     /* Eject disc */
     ret = _cdio_mciSendCommand( op.wDeviceID, MCI_SET, command_flags, 0 ) == 0;
     /* Release access to the device */
@@ -618,6 +645,23 @@ eject_media_win32 (void *p_user_data)
   return open_close_media_win32(psz_drive, MCI_SET_DOOR_OPEN);
 }
 
+static bool
+is_mmc_supported(void *user_data)
+{
+    _img_private_t *env = user_data;
+    switch (env->access_mode) {
+      case _AM_NONE:
+	return false;
+      case _AM_IOCTL:
+      case _AM_ASPI:
+      case _AM_MMC_RDWR:
+      case _AM_MMC_RDWR_EXCL:
+	return true;
+    }
+    /* Not reached. */
+    return false;
+}
+
 /*!
   Return the value associated with the key "arg".
 */
@@ -629,11 +673,23 @@ get_arg_win32 (void *p_user_data, const char key[])
   if (!strcmp (key, "source")) {
     return p_env->gen.source_name;
   } else if (!strcmp (key, "access-mode")) {
-    if (p_env->hASPI) 
-      return "ASPI";
-    else 
+    switch (p_env->access_mode) {
+    case _AM_IOCTL:
       return "ioctl";
-  } 
+    case _AM_ASPI:
+      return "ASPI";
+    case _AM_MMC_RDWR:
+      return "MMC_RDWR";
+    case _AM_MMC_RDWR_EXCL:
+      return "MMC_RDWR_EXCL";
+    case _AM_NONE:
+      return "no access method";
+    }
+  } else if (!strcmp (key, "scsi-tuple")) {
+    return p_env->gen.scsi_tuple;
+  } else if (!strcmp (key, "mmc-supported?")) {
+      return is_mmc_supported(p_user_data) ? "true" : "false";
+  }
   return NULL;
 }
 
@@ -808,18 +864,19 @@ cdio_get_default_device_win32(void)
 }
 
 /*!  
-  Return true if source_name could be a device containing a CD-ROM.
+  Return true if source_name could be a device containing a CD-ROM and
+  we are on a MS Windows platform.
 */
 bool
 cdio_is_device_win32(const char *source_name)
 {
+#ifdef HAVE_WIN32_CDROM
   unsigned int len;
 
-  len = strlen(source_name);
   if (NULL == source_name) return false;
+  len = strlen(source_name);
 
-#ifdef HAVE_WIN32_CDROM
-  if ((len == 2) && isalpha(source_name[0]) 
+  if ((len == 2) && isalpha((unsigned char) source_name[0]) 
 	    && (source_name[len-1] == ':'))
     return true;
 
@@ -829,7 +886,7 @@ cdio_is_device_win32(const char *source_name)
   return ( (len == 6) 
 	   && source_name[0] == '\\' && source_name[1] == '\\'
 	   && source_name[2] == '.'  && source_name[3] == '\\'
-	   && isalpha(source_name[len-2])
+	   && isalpha((unsigned char) source_name[len-2])
 	   && (source_name[len-1] == ':') );
 #else 
   return false;
@@ -905,6 +962,7 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
   _funcs.free                   = free_win32;
   _funcs.get_arg                = get_arg_win32;
   _funcs.get_cdtext             = get_cdtext_generic;
+  _funcs.get_cdtext_raw         = read_cdtext_generic;
   _funcs.get_default_device     = cdio_get_default_device_win32;
   _funcs.get_devices            = cdio_get_devices_win32;
   _funcs.get_disc_last_lsn      = get_disc_last_lsn_win32;
@@ -925,20 +983,16 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
   _funcs.lseek                  = NULL;
   _funcs.read                   = NULL;
   _funcs.read_audio_sectors     = read_audio_sectors;
-#if 0
-  _funcs.read_data_sectors      = read_data_sectors_generic;
-#else
-  _funcs.read_data_sectors      = read_data_sectors_mmc;
-#endif
-  _funcs.read_mode1_sector      = read_mode1_sector;
-  _funcs.read_mode1_sectors     = read_mode1_sectors;
-  _funcs.read_mode2_sector      = read_mode2_sector;
-  _funcs.read_mode2_sectors     = read_mode2_sectors;
+  _funcs.read_data_sectors      = read_data_sectors_win32;
+  _funcs.read_mode1_sector      = read_mode1_sector_win32;
+  _funcs.read_mode1_sectors     = read_mode1_sectors_win32;
+  _funcs.read_mode2_sector      = read_mode2_sector_win32;
+  _funcs.read_mode2_sectors     = read_mode2_sectors_win32;
   _funcs.read_toc               = read_toc_win32;
   _funcs.run_mmc_cmd            = run_mmc_cmd_win32;
   _funcs.set_arg                = set_arg_win32;
   _funcs.set_blocksize          = set_blocksize_mmc;
-  _funcs.set_speed              = set_speed_mmc;
+  _funcs.set_speed              = set_drive_speed_mmc;
 
   _data                 = calloc(1, sizeof (_img_private_t));
   _data->access_mode    = str_to_access_mode_win32(psz_access_mode);
@@ -954,10 +1008,9 @@ cdio_open_am_win32 (const char *psz_orig_source, const char *psz_access_mode)
     if (cdio_is_device_win32(psz_orig_source))
       set_arg_win32(_data, "source", psz_orig_source);
     else {
-      /* The below would be okay if all device drivers worked this way. */
-#if 0
-      cdio_info ("source %s is a not a device", psz_orig_source);
-#endif
+      /* The below would be okay as an info message if all device
+	 drivers worked this way. */
+      cdio_debug ("source %s is a not a device", psz_orig_source);
       free(_data);
       return NULL;
     }
