@@ -1,5 +1,6 @@
 /*
-  Copyright (C) 2004, 2005, 2008, 2010, 2011 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2004, 2005, 2008, 2010, 2011, 2012
+  Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,12 +31,28 @@
 # include "NtScsi.h"
 # include "undocumented.h"
 #else
-# include <ddk/ntddcdrm.h>
-# include <ddk/ntddscsi.h>
-# include <ddk/scsi.h>
+# if defined (__MINGW64__)
+#  define _NTSRB_ /* Bad things happen if srb.h gets included */
+# endif
+# include <windows.h>
+# ifdef HAVE_DDK_SCSI_H
+#  include <ddk/scsi.h>
+# endif
+# ifdef HAVE_NTDDCDRM_H
+#  include <ntddcdrm.h>
+# endif
+# ifdef HAVE_DDK_NTDDCDRM_H
+#  include <ddk/ntddcdrm.h>
+# endif
+# ifdef HAVE_NTDDSCSI_H
+#  include <ntddscsi.h>
+# endif
+# ifdef HAVE_DDK_NTDDSCSI_H
+#  include <ddk/ntddscsi.h>
+# endif
 #endif
 
-#ifdef WIN32
+#if defined (_WIN32)
 #include <windows.h>
 #endif
 
@@ -50,7 +67,6 @@
 #include <cdio/util.h>
 #include "cdio_assert.h"
 #include <cdio/mmc.h>
-#include "cdtext_private.h"
 #include "cdio/logging.h"
 
 #if defined (_XBOX)
@@ -100,21 +116,26 @@ typedef struct _CDROM_TOC_FULL {
     TRACK_DATA_FULL TrackData[CDIO_CD_MAX_TRACKS+3];
 } CDROM_TOC_FULL, *PCDROM_TOC_FULL;
 
+#define SPT_CDB_LENGTH 32
+#define SPT_SENSE_LENGTH 32
+#define SPTWB_DATA_LENGTH 512
+
 typedef struct {
    SCSI_PASS_THROUGH_DIRECT sptd;
    ULONG Filler; /* Realign buffer to double-word boundary */
-   cdio_mmc_request_sense_t   SenseBuf;
-   UCHAR DataBuf[512];
+   UCHAR SenseBuf[SPT_SENSE_LENGTH];
 } SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER;
 
-typedef struct _SCSI_PASS_THROUGH_WITH_BUFFER {
+typedef struct _SCSI_PASS_THROUGH_WITH_BUFFERS {
     SCSI_PASS_THROUGH    Spt;
     ULONG                Filler;      /* realign buffer to double-word boundary */
-    cdio_mmc_request_sense_t  SenseBuf;
-    UCHAR                DataBuf[512];
-} SCSI_PASS_THROUGH_WITH_BUFFER;
-
+    UCHAR                SenseBuf[SPT_SENSE_LENGTH];
+    UCHAR DataBuf[SPTWB_DATA_LENGTH];
+} SCSI_PASS_THROUGH_WITH_BUFFERS;
 
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif 
 
 #include "win32.h"
 
@@ -440,7 +461,7 @@ set_scsi_tuple_win32ioctl(_img_private_t *env)
 
   Return 0 if command completed successfully.
  */
-#if 1
+#if USE_PASSTHROUGH_DIRECT
 int
 run_mmc_cmd_win32ioctl( void *p_user_data, 
                         unsigned int i_timeout_ms,
@@ -456,6 +477,7 @@ run_mmc_cmd_win32ioctl( void *p_user_data,
   char dummy_buf[2]; /* Used if we can't use p_buf. See below. */
   int rc = DRIVER_OP_SUCCESS;
 
+  p_env->gen.scsi_mmc_sense_valid = 0;
   memset(&swb, 0, sizeof(swb));
   
   swb.sptd.Length  = sizeof(SCSI_PASS_THROUGH_DIRECT);
@@ -514,19 +536,24 @@ run_mmc_cmd_win32ioctl( void *p_user_data,
     }
   }
 
+#ifdef FIXED_ADDITIONAL_SENSE_BUF
   /* Record SCSI sense reply for API call mmc_last_cmd_sense(). 
    */
   if (swb.SenseBuf.additional_sense_len) {
     /* SCSI Primary Command standard 
        SPC 4.5.3, Table 26: 252 bytes legal, 263 bytes possible */
     int sense_size = swb.SenseBuf.additional_sense_len + 8; 
-    if (sense_size > sizeof(swb.SenseBuf))
+    if (sense_size > sizeof(swb.SenseBuf)) {
+      cdio_warn("Sense size retuned %d is greater buffer size %d\n", 
+		sense_size, sizeof(swb.SenseBuf));
       sense_size = sizeof(swb.SenseBuf);
+    }
     memcpy((void *) p_env->gen.scsi_mmc_sense, &swb.SenseBuf, sense_size);
     p_env->gen.scsi_mmc_sense_valid = sense_size;
     if (DRIVER_OP_SUCCESS == rc) 
       rc = DRIVER_OP_MMC_SENSE_DATA;
   }
+#endif
   return rc;
 }
 #else
@@ -538,12 +565,13 @@ run_mmc_cmd_win32ioctl( void *p_user_data,
                         unsigned int i_buf, /*in/out*/ void *p_buf )
 {
   _img_private_t *p_env = p_user_data;
-  SCSI_PASS_THROUGH_WITH_BUFFER sptwb;
+  SCSI_PASS_THROUGH_WITH_BUFFERS sptwb;
   
   BOOL b_success;
   unsigned long length = 0;
   DWORD dw_bytes_returned;
 
+  p_env->gen.scsi_mmc_sense_valid = 0;
   memset(&sptwb, 0, sizeof(sptwb));
   
   sptwb.Spt.Length  = sizeof(sptwb.Spt);
@@ -564,13 +592,13 @@ run_mmc_cmd_win32ioctl( void *p_user_data,
   sptwb.Spt.TimeOutValue      = msecs2secs(i_timeout_ms);
 
   sptwb.Spt.DataBufferOffset =
-    offsetof(SCSI_PASS_THROUGH_WITH_BUFFER,DataBuf);
+    offsetof(SCSI_PASS_THROUGH_WITH_BUFFERS, DataBuf);
   sptwb.Spt.SenseInfoOffset = 
-    offsetof(SCSI_PASS_THROUGH_WITH_BUFFER, SenseBuf);
+    offsetof(SCSI_PASS_THROUGH_WITH_BUFFERS, SenseBuf);
 
   memcpy(sptwb.Spt.Cdb, p_cdb, i_cdb);
   sptwb.Spt.Cdb[4] = i_buf;
-  length = offsetof(SCSI_PASS_THROUGH_WITH_BUFFER,DataBuf) + 
+  length = offsetof(SCSI_PASS_THROUGH_WITH_BUFFERS, DataBuf) + 
     sptwb.Spt.DataTransferLength;
 
   /* Send the command to drive */
@@ -584,7 +612,11 @@ run_mmc_cmd_win32ioctl( void *p_user_data,
                               NULL);
 
   if ( 0 == b_success ) {
+    char buffer[100];
+    snprintf(buffer, sizeof(buffer), 
+	     "MMC command code: 0x%x\n", p_cdb->field[0]);
     windows_error(CDIO_LOG_INFO, GetLastError());
+    cdio_log(CDIO_LOG_INFO, buffer);
     return DRIVER_OP_ERROR;
   }
 
@@ -592,14 +624,15 @@ run_mmc_cmd_win32ioctl( void *p_user_data,
 
   /* Record SCSI sense reply for API call mmc_last_cmd_sense(). 
    */
-  if (sptwb.SenseBuf[7]) {
-    int sense_size = sptwb.SenseBuf[7] + 8; /* SPC 4.5.3, Table 26: 
-                                               252 bytes legal, 263 bytes
-                                               possible */
-    if (sense_size > sizeof(sptwb.SenseBuf))
+  if (sptwb.Spt.ScsiStatus && sptwb.Spt.SenseInfoLength > 0) {
+    int sense_size = sptwb.Spt.SenseInfoLength;
+    if (sense_size > sizeof(sptwb.SenseBuf)) {
+      cdio_warn("sense size returned %d is greater buffer size %d\n", 
+		sense_size, sizeof(sptwb.SenseBuf));
       sense_size = sizeof(sptwb.SenseBuf);
+    }
     memcpy((void *) p_env->gen.scsi_mmc_sense, &sptwb.SenseBuf, sense_size);
-    p_env->gen.scsi_mmc_sense_valid = sense_size;
+    p_env->gen.scsi_mmc_sense_valid = sptwb.Spt.SenseInfoLength;
   }
 
   return DRIVER_OP_SUCCESS;
@@ -1091,7 +1124,11 @@ read_toc_win32ioctl (_img_private_t *p_env)
     cdio_log(loglevel, "could not read TOC");
     windows_error(loglevel, GetLastError());
 
+#ifdef RUN_MMC_CMD_WIN32IOCTL_FULLY_FIXED
+    /* rocky: there is some brokenness in run_mmc_cmd_win32ioctl. So until that
+       is fixed, not running the below will mitigate that somewhat. */
     if ( !b_fulltoc_first && read_fulltoc_win32mmc(p_env) ) return true;
+#endif
     return false;
   }
   
