@@ -1,5 +1,6 @@
 /*
-  Copyright (C) 2004, 2005, 2006, 2008, 2009 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2004, 2005, 2006, 2008, 2009, 2012
+  Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -18,8 +19,8 @@
 /* Internal routines for CD I/O drivers. */
 
 
-#ifndef __CDIO_GENERIC_H__
-#define __CDIO_GENERIC_H__
+#ifndef CDIO_DRIVER_GENERIC_H_
+#define CDIO_DRIVER_GENERIC_H_
 
 #if defined(HAVE_CONFIG_H) && !defined(LIBCDIO_CONFIG_H)
 # include "config.h"
@@ -29,6 +30,10 @@
 #include <cdio/cdtext.h>
 #include <cdio/iso9660.h>
 
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif 
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -43,7 +48,6 @@ extern "C" {
     char *source_name;      /**< Name used in open. */
     bool  init;             /**< True if structure has been initialized */
     bool  toc_init;         /**< True if TOC read in */
-    bool  b_cdtext_init;    /**< True if CD-Text read in */
     bool  b_cdtext_error;   /**< True if trouble reading CD-Text */
     
     int   ioctls_debugged;  /**< for debugging */
@@ -58,12 +62,11 @@ extern "C" {
     track_t i_tracks;       /**< The number of tracks. */
 
     uint8_t i_joliet_level; /**< 0 = no Joliet extensions.
-			       1-3: Joliet level. */
+                               1-3: Joliet level. */
     iso9660_pvd_t pvd;      
     iso9660_svd_t svd;      
     CdIo_t   *cdio;         /**< a way to call general cdio routines. */
-    cdtext_t  cdtext;       /**< CD-Text for disc. */
-    cdtext_t  cdtext_track[CDIO_CD_MAX_TRACKS+1]; /**< CD-TEXT for each track*/
+    cdtext_t *cdtext;       /**< CD-Text for disc. */
     track_flags_t track_flags[CDIO_CD_MAX_TRACKS+1];
 
     /* Memorized sense reply of the most recent SCSI command.
@@ -177,7 +180,7 @@ extern "C" {
     @return the CD-TEXT object or NULL if obj is NULL
     or CD-TEXT information does not exist.
   */
-  cdtext_t *get_cdtext_generic (void *p_user_data, track_t i_track);
+  cdtext_t *get_cdtext_generic (void *p_user_data);
 
   /*!
     Return the number of of the first track. 
@@ -210,7 +213,7 @@ extern "C" {
     Is this meaningful if not an audio track?
   */
   track_flag_t get_track_copy_permit_generic(void *p_user_data, 
-					     track_t i_track);
+                                             track_t i_track);
   
   /*! Return 1 if track has pre-emphasis, 0 if not, or -1 for error.
     Is this meaningful if not an audio track?
@@ -218,31 +221,28 @@ extern "C" {
     pre-emphasis is a non linear frequency response.
   */
   track_flag_t get_track_preemphasis_generic(const void *p_user_data, 
-					     track_t i_track);
+                                             track_t i_track);
   
-  void set_cdtext_field_generic(void *user_data, track_t i_track, 
-				track_t i_first_track,
-				cdtext_field_t e_field, const char *psz_value);
   /*!
     Read cdtext information for a CdIo object .
   
     return true on success, false on error or CD-Text information does
     not exist.
   */
-  bool init_cdtext_generic (generic_img_private_t *p_env);
+  uint8_t * read_cdtext_generic (void *p_env);
   
   void set_track_flags(track_flags_t *p_track_flag, uint8_t flag);
   
   /*! Read mode 1 or mode2 sectors (using cooked mode).  */
   driver_return_code_t read_data_sectors_generic (void *p_user_data, 
-						  void *p_buf, lsn_t i_lsn, 
-						  uint16_t i_blocksize, 
-						  uint32_t i_blocks);
+                                                  void *p_buf, lsn_t i_lsn, 
+                                                  uint16_t i_blocksize, 
+                                                  uint32_t i_blocks);
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* __CDIO_GENERIC_H__ */
+#endif /* CDIO_DRIVER_GENERIC_H_ */
 
 
 /* 
