@@ -1,9 +1,7 @@
 /*
-  $Id: gnu_linux.c,v 1.33 2008/06/25 07:46:21 rocky Exp $
-
   Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2008, 2009
-    Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2008, 2009, 2010, 2011
+  2012 Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,10 +23,9 @@
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 
-static const char _rcsid[] = "$Id: gnu_linux.c,v 1.33 2008/06/25 07:46:21 rocky Exp $";
-
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -36,10 +33,9 @@ static const char _rcsid[] = "$Id: gnu_linux.c,v 1.33 2008/06/25 07:46:21 rocky
 #include <cdio/sector.h>
 #include <cdio/util.h>
 #include <cdio/types.h>
-#include <cdio/mmc.h>
+#include <cdio/mmc_cmds.h>
 #include <cdio/audio.h>
 #include <cdio/cdtext.h>
-#include "cdtext_private.h"
 #include "cdio_assert.h"
 #include "cdio_private.h"
 
@@ -85,7 +81,9 @@ typedef enum {
   _AM_NONE,
   _AM_IOCTL,
   _AM_READ_CD,
-  _AM_READ_10
+  _AM_READ_10,
+  _AM_MMC_RDWR,
+  _AM_MMC_RDWR_EXCL,
 } access_mode_t;
 
 typedef struct {
@@ -104,8 +102,8 @@ typedef struct {
 } _img_private_t;
 
 /* Some ioctl() errno values which occur when the tray is empty */
-#define ERRNO_TRAYEMPTY(errno)	\
-	((errno == EIO) || (errno == ENOENT) || (errno == EINVAL))
+#define ERRNO_TRAYEMPTY(errno)  \
+        ((errno == EIO) || (errno == ENOENT) || (errno == EINVAL))
 
 /**** prototypes for static functions ****/
 static bool is_cdrom_linux(const char *drive, char *mnttype);
@@ -132,9 +130,13 @@ str_to_access_mode_linux(const char *psz_access_mode)
     return _AM_READ_CD;
   else if (!strcmp(psz_access_mode, "READ_10"))
     return _AM_READ_10;
+  else if (!strcmp(psz_access_mode, "MMC_RDWR"))
+    return _AM_MMC_RDWR;
+  else if (!strcmp(psz_access_mode, "MMC_RDWR_EXCL"))
+    return _AM_MMC_RDWR_EXCL;
   else {
     cdio_warn ("unknown access type: %s. Default IOCTL used.", 
-	       psz_access_mode);
+               psz_access_mode);
     return default_access_mode;
   }
 }
@@ -157,13 +159,13 @@ check_mounts_linux(const char *mtab)
       i_mnt_type = strlen(mntent->mnt_type) + 1;
       mnt_type = calloc(1, i_mnt_type);
       if (mnt_type == NULL)
-	continue;  /* maybe you'll get lucky next time. */
+        continue;  /* maybe you'll get lucky next time. */
       
       i_mnt_dev = strlen(mntent->mnt_fsname) + 1;
       mnt_dev = calloc(1, i_mnt_dev);
       if (mnt_dev == NULL) {
-	free(mnt_type);
-	continue;
+        free(mnt_type);
+        continue;
       }
       
       strncpy(mnt_type, mntent->mnt_type, i_mnt_type);
@@ -171,35 +173,37 @@ check_mounts_linux(const char *mtab)
       
       /* Handle "supermount" filesystem mounts */
       if ( strcmp(mnt_type, "supermount") == 0 ) {
-	tmp = strstr(mntent->mnt_opts, "fs=");
-	if ( tmp ) {
-	  free(mnt_type);
-	  mnt_type = strdup(tmp + strlen("fs="));
-	  if ( mnt_type ) {
-	    tmp = strchr(mnt_type, ',');
-	    if ( tmp ) {
-	      *tmp = '\0';
-	    }
-	  }
-	}
-	tmp = strstr(mntent->mnt_opts, "dev=");
-	if ( tmp ) {
-	  free(mnt_dev);
-	  mnt_dev = strdup(tmp + strlen("dev="));
-	  if ( mnt_dev ) {
-	    tmp = strchr(mnt_dev, ',');
-	    if ( tmp ) {
-	      *tmp = '\0';
-	    }
-	  }
-	}
+        tmp = strstr(mntent->mnt_opts, "fs=");
+        if ( tmp ) {
+          free(mnt_type);
+          mnt_type = strdup(tmp + strlen("fs="));
+          if ( mnt_type ) {
+            tmp = strchr(mnt_type, ',');
+            if ( tmp ) {
+              *tmp = '\0';
+            }
+          }
+        }
+        tmp = strstr(mntent->mnt_opts, "dev=");
+        if ( tmp ) {
+          free(mnt_dev);
+          mnt_dev = strdup(tmp + strlen("dev="));
+          if ( mnt_dev ) {
+            tmp = strchr(mnt_dev, ',');
+            if ( tmp ) {
+              *tmp = '\0';
+            }
+          }
+        }
       }
-      if ( strcmp(mnt_type, "iso9660") == 0 ) {
-	if (is_cdrom_linux(mnt_dev, mnt_type) > 0) {
-	  free(mnt_type);
-	  endmntent(mntfp);
-	  return mnt_dev;
-	}
+      if ( mnt_type && mnt_dev ) {
+	if ( strcmp(mnt_type, "iso9660") == 0 ) {
+	  if (is_cdrom_linux(mnt_dev, mnt_type) > 0) {
+	    free(mnt_type);
+	    endmntent(mntfp);
+	    return mnt_dev;
+	  }
+        }
       }
       free(mnt_dev);
       free(mnt_type);
@@ -356,6 +360,13 @@ audio_stop_linux (void *p_user_data)
   return ioctl(p_env->gen.fd, CDROMSTOP);
 }
 
+static bool
+is_mmc_supported(void *user_data)
+{
+    _img_private_t *env = user_data;
+    return (_AM_NONE == env->access_mode) ? false : true;
+}
+
 /*!
   Return the value associated with the key "arg".
 */
@@ -369,14 +380,22 @@ get_arg_linux (void *env, const char key[])
   } else if (!strcmp (key, "access-mode")) {
     switch (_obj->access_mode) {
     case _AM_IOCTL:
-      return "ioctl";
+      return "IOCTL";
     case _AM_READ_CD:
       return "READ_CD";
     case _AM_READ_10:
       return "READ_10";
+    case _AM_MMC_RDWR:
+      return "MMC_RDWR";
+    case _AM_MMC_RDWR_EXCL:
+      return "MMC_RDWR_EXCL";
     case _AM_NONE:
       return "no access method";
     }
+  } else if (!strcmp (key, "scsi-tuple")) {
+    return _obj->gen.scsi_tuple;
+  } else if (!strcmp (key, "mmc-supported?")) {
+      return is_mmc_supported(env) ? "true" : "false";
   } 
   return NULL;
 }
@@ -392,9 +411,9 @@ get_arg_linux (void *env, const char key[])
  */
 static void
 get_drive_cap_linux (const void *p_user_data,
-		     /*out*/ cdio_drive_read_cap_t  *p_read_cap,
-		     /*out*/ cdio_drive_write_cap_t *p_write_cap,
-		     /*out*/ cdio_drive_misc_cap_t  *p_misc_cap)
+                     /*out*/ cdio_drive_read_cap_t  *p_read_cap,
+                     /*out*/ cdio_drive_write_cap_t *p_write_cap,
+                     /*out*/ cdio_drive_misc_cap_t  *p_misc_cap)
 {
   const _img_private_t *p_env = p_user_data;
   int32_t i_drivetype;
@@ -581,7 +600,9 @@ get_track_msf_linux(void *p_user_data, track_t i_track, msf_t *msf)
 {
   _img_private_t *p_env = p_user_data;
 
-  if (NULL == msf) return false;
+  if (NULL == msf || 
+      (i_track > CDIO_CD_MAX_TRACKS && i_track != CDIO_CDROM_LEADOUT_TRACK))
+    return false;
 
   if (!p_env->gen.toc_init) read_toc_linux (p_user_data) ;
 
@@ -624,12 +645,18 @@ static int is_mounted (const char * device, char * target) {
   if(!fp) return 0;
 
   /* Get real device */
-  cdio_follow_symlink(device, real_device_1);
+  if (NULL == cdio_realpath(device, real_device_1)) {
+    cdio_warn("Problems resolving device %s: %s\n", device, strerror(errno));
+  }
+    
     
   /* Read entries */
 
   while ( fscanf(fp, "%s %s %*s %*s %*d %*d\n", file_device, file_target) != EOF ) {
-    cdio_follow_symlink(file_device, real_device_2);
+      if (NULL == cdio_realpath(file_device, real_device_2)) {
+          cdio_debug("Problems resolving device %s: %s\n", 
+                     file_device, strerror(errno));
+      }
     if(!strcmp(real_device_1, real_device_2)) {
       strcpy(target, file_target);
       fclose(fp);
@@ -660,10 +687,8 @@ static int do_umount(char * target) {
     execlp("pumount", "pumount", target, NULL);
     execlp("umount", "umount", target, NULL);
     return -1;
-    break;
   case -1:
     return -1;
-    break;
   default: /* parent */
     wait(&status);
     if (WIFEXITED(status) == 0) {
@@ -818,7 +843,7 @@ get_discmode_linux (void *p_user_data)
       return CDIO_DISC_MODE_CD_DA;
     case CDS_DATA_1:
     case CDS_DATA_2: /* Actually, recent GNU/Linux kernels don't return 
-			CDS_DATA_2, but just in case. */
+                        CDS_DATA_2, but just in case. */
       return CDIO_DISC_MODE_CD_DATA;
     case CDS_MIXED:
       return CDIO_DISC_MODE_CD_MIXED;
@@ -868,7 +893,7 @@ is_cdrom_linux(const char *drive, char *mnttype)
 */
 static driver_return_code_t
 read_audio_sectors_linux (void *p_user_data, void *p_buf, lsn_t i_lsn, 
-			   uint32_t i_blocks)
+                           uint32_t i_blocks)
 {
   _img_private_t *p_env = p_user_data;
   return mmc_read_sectors( p_env->gen.cdio, p_buf, i_lsn, 
@@ -880,7 +905,7 @@ read_audio_sectors_linux (void *p_user_data, void *p_buf, lsn_t i_lsn,
 */
 static driver_return_code_t
 _read_mode2_sectors_mmc (_img_private_t *p_env, void *p_buf, lba_t lba, 
-			 uint32_t i_blocks, bool b_read_10)
+                         uint32_t i_blocks, bool b_read_10)
 {
   mmc_cdb_t cdb = {{0, }};
 
@@ -902,8 +927,8 @@ _read_mode2_sectors_mmc (_img_private_t *p_env, void *p_buf, lba_t lba,
                                      M2RAW_SECTOR_SIZE * i_blocks, 
                                      p_buf)))
       {
-	mmc_set_blocksize (p_env->gen.cdio, CDIO_CD_FRAMESIZE);
-	return retval;
+        mmc_set_blocksize (p_env->gen.cdio, CDIO_CD_FRAMESIZE);
+        return retval;
       }
     
     /* Restore blocksize. */
@@ -926,7 +951,7 @@ _read_mode2_sectors_mmc (_img_private_t *p_env, void *p_buf, lba_t lba,
 
 static driver_return_code_t
 _read_mode2_sectors (_img_private_t *p_env, void *p_buf, lba_t lba, 
-		     uint32_t i_blocks, bool b_read_10)
+                     uint32_t i_blocks, bool b_read_10)
 {
   unsigned int l = 0;
   int retval = 0;
@@ -937,10 +962,10 @@ _read_mode2_sectors (_img_private_t *p_env, void *p_buf, lba_t lba,
       void *p_buf2 = ((char *)p_buf ) + (l * M2RAW_SECTOR_SIZE);
       
       retval |= _read_mode2_sectors_mmc (p_env, p_buf2, lba + l, 
-					 i_blocks2, b_read_10);
+                                         i_blocks2, b_read_10);
 
       if (retval)
-	break;
+        break;
 
       i_blocks -= i_blocks2;
       l += i_blocks2;
@@ -955,7 +980,7 @@ _read_mode2_sectors (_img_private_t *p_env, void *p_buf, lba_t lba,
  */
 static driver_return_code_t
 _read_mode1_sector_linux (void *p_user_data, void *p_data, lsn_t lsn, 
-			 bool b_form2)
+                          bool b_form2)
 {
 
 #if 0
@@ -980,38 +1005,38 @@ _read_mode1_sector_linux (void *p_user_data, void *p_data, lsn_t lsn,
       
     case _AM_IOCTL:
       if (ioctl (p_env->gen.fd, CDROMREADMODE1, &buf) == -1)
-	{
-	  perror ("ioctl()");
-	  return 1;
-	  /* exit (EXIT_FAILURE); */
-	}
+        {
+          perror ("ioctl()");
+          return 1;
+          /* exit (EXIT_FAILURE); */
+        }
       break;
       
     case _AM_READ_CD:
     case _AM_READ_10:
       if (_read_mode2_sectors (p_env, buf, lsn, 1, 
                                (p_env->access_mode == _AM_READ_10)))
-	{
-	  perror ("ioctl()");
-	  if (p_env->access_mode == _AM_READ_CD)
-	    {
-	      cdio_info ("READ_CD failed; switching to READ_10 mode...");
-	      p_env->access_mode = _AM_READ_10;
-	      goto retry;
-	    }
-	  else
-	    {
-	      cdio_info ("READ_10 failed; switching to ioctl(CDROMREADMODE2) mode...");
-	      p_env->access_mode = _AM_IOCTL;
-	      goto retry;
-	    }
-	  return 1;
-	}
+        {
+          perror ("ioctl()");
+          if (p_env->access_mode == _AM_READ_CD)
+            {
+              cdio_info ("READ_CD failed; switching to READ_10 mode...");
+              p_env->access_mode = _AM_READ_10;
+              goto retry;
+            }
+          else
+            {
+              cdio_info ("READ_10 failed; switching to ioctl(CDROMREADMODE2) mode...");
+              p_env->access_mode = _AM_IOCTL;
+              goto retry;
+            }
+          return 1;
+        }
       break;
     }
 
   memcpy (p_data, buf + CDIO_CD_SYNC_SIZE + CDIO_CD_HEADER_SIZE, 
-	  b_form2 ? M2RAW_SECTOR_SIZE: CDIO_CD_FRAMESIZE);
+          b_form2 ? M2RAW_SECTOR_SIZE: CDIO_CD_FRAMESIZE);
   
 #else
   return cdio_generic_read_form1_sector(p_user_data, p_data, lsn);
@@ -1026,7 +1051,7 @@ _read_mode1_sector_linux (void *p_user_data, void *p_data, lsn_t lsn,
  */
 static driver_return_code_t
 _read_mode1_sectors_linux (void *p_user_data, void *p_data, lsn_t lsn, 
-			  bool b_form2, uint32_t i_blocks)
+                           bool b_form2, uint32_t i_blocks)
 {
   _img_private_t *p_env = p_user_data;
   unsigned int i;
@@ -1035,8 +1060,8 @@ _read_mode1_sectors_linux (void *p_user_data, void *p_data, lsn_t lsn,
 
   for (i = 0; i < i_blocks; i++) {
     if ( (retval = _read_mode1_sector_linux (p_env,
-					    ((char *)p_data) + (blocksize*i),
-					    lsn + i, b_form2)) )
+                                            ((char *)p_data) + (blocksize*i),
+                                            lsn + i, b_form2)) )
       return retval;
   }
   return DRIVER_OP_SUCCESS;
@@ -1048,7 +1073,7 @@ _read_mode1_sectors_linux (void *p_user_data, void *p_data, lsn_t lsn,
  */
 static driver_return_code_t
 _read_mode2_sector_linux (void *p_user_data, void *p_data, lsn_t lsn, 
-			  bool b_form2)
+                          bool b_form2)
 {
   char buf[M2RAW_SECTOR_SIZE] = { 0, };
   struct cdrom_msf *msf = (struct cdrom_msf *) &buf;
@@ -1070,34 +1095,36 @@ _read_mode2_sector_linux (void *p_user_data, void *p_data, lsn_t lsn,
       break;
       
     case _AM_IOCTL:
+    case _AM_MMC_RDWR:
+    case _AM_MMC_RDWR_EXCL:
       if (ioctl (p_env->gen.fd, CDROMREADMODE2, &buf) == -1)
-	{
-	  perror ("ioctl()");
-	  return 1;
-	  /* exit (EXIT_FAILURE); */
-	}
+        {
+          perror ("ioctl()");
+          return 1;
+          /* exit (EXIT_FAILURE); */
+        }
       break;
       
     case _AM_READ_CD:
     case _AM_READ_10:
       if (_read_mode2_sectors (p_env, buf, lsn, 1, 
-			       (p_env->access_mode == _AM_READ_10)))
-	{
-	  perror ("ioctl()");
-	  if (p_env->access_mode == _AM_READ_CD)
-	    {
-	      cdio_info ("READ_CD failed; switching to READ_10 mode...");
-	      p_env->access_mode = _AM_READ_10;
-	      goto retry;
-	    }
-	  else
-	    {
-	      cdio_info ("READ_10 failed; switching to ioctl(CDROMREADMODE2) mode...");
-	      p_env->access_mode = _AM_IOCTL;
-	      goto retry;
-	    }
-	  return 1;
-	}
+                               (p_env->access_mode == _AM_READ_10)))
+        {
+          perror ("ioctl()");
+          if (p_env->access_mode == _AM_READ_CD)
+            {
+              cdio_info ("READ_CD failed; switching to READ_10 mode...");
+              p_env->access_mode = _AM_READ_10;
+              goto retry;
+            }
+          else
+            {
+              cdio_info ("READ_10 failed; switching to ioctl(CDROMREADMODE2) mode...");
+              p_env->access_mode = _AM_IOCTL;
+              goto retry;
+            }
+          return 1;
+        }
       break;
     }
 
@@ -1116,7 +1143,7 @@ _read_mode2_sector_linux (void *p_user_data, void *p_data, lsn_t lsn,
  */
 static driver_return_code_t
 _read_mode2_sectors_linux (void *p_user_data, void *data, lsn_t lsn, 
-			  bool b_form2, uint32_t i_blocks)
+                           bool b_form2, uint32_t i_blocks)
 {
   _img_private_t *p_env = p_user_data;
   unsigned int i;
@@ -1126,8 +1153,8 @@ _read_mode2_sectors_linux (void *p_user_data, void *data, lsn_t lsn,
   for (i = 0; i < i_blocks; i++) {
     int retval;
     if ( (retval = _read_mode2_sector_linux (p_env, 
-					    ((char *)data) + (i_blocksize*i),
-					    lsn + i, b_form2)) )
+                                            ((char *)data) + (i_blocksize*i),
+                                            lsn + i, b_form2)) )
       return retval;
   }
   return DRIVER_OP_SUCCESS;
@@ -1153,6 +1180,13 @@ read_toc_linux (void *p_user_data)
   p_env->gen.i_first_track = p_env->tochdr.cdth_trk0;
   p_env->gen.i_tracks      = p_env->tochdr.cdth_trk1;
 
+  if (p_env->gen.i_tracks > CDIO_CD_MAX_TRACKS) {
+     cdio_log(CDIO_LOG_WARN, "Number of tracks exceeds maximum (%d vs. %d)\n",
+              p_env->gen.i_tracks, CDIO_CD_MAX_TRACKS);
+     p_env->gen.i_tracks = CDIO_CD_MAX_TRACKS;
+  }
+
+
   /* read individual tracks */
   for (i= p_env->gen.i_first_track; i<=p_env->gen.i_tracks; i++) {
     struct cdrom_tocentry *p_toc = 
@@ -1173,7 +1207,7 @@ read_toc_linux (void *p_user_data)
     struct cdrom_msf0 *msf= &env->tocent[i-1].cdte_addr.msf;
     
     fprintf (stdout, "--- track# %d (msf %2.2x:%2.2x:%2.2x)\n",
-	     i, msf->minute, msf->second, msf->frame);
+             i, msf->minute, msf->second, msf->frame);
     ****/
 
   }
@@ -1183,9 +1217,9 @@ read_toc_linux (void *p_user_data)
   p_env->tocent[p_env->gen.i_tracks].cdte_format = CDROM_MSF;
 
   if (ioctl(p_env->gen.fd, CDROMREADTOCENTRY, 
-	    &p_env->tocent[p_env->gen.i_tracks]) == -1 ) {
+            &p_env->tocent[p_env->gen.i_tracks]) == -1 ) {
     cdio_warn("%s: %s\n", 
-	     "error in ioctl CDROMREADTOCENTRY for lead-out",
+             "error in ioctl CDROMREADTOCENTRY for lead-out",
             strerror(errno));
     return false;
   }
@@ -1194,7 +1228,7 @@ read_toc_linux (void *p_user_data)
   struct cdrom_msf0 *msf= &env->tocent[p_env->gen.i_tracks].cdte_addr.msf;
 
   fprintf (stdout, "--- track# %d (msf %2.2x:%2.2x:%2.2x)\n",
-	   i, msf->minute, msf->second, msf->frame);
+           i, msf->minute, msf->second, msf->frame);
   */
 
   p_env->gen.toc_init = true;
@@ -1204,43 +1238,68 @@ read_toc_linux (void *p_user_data)
 /*!
   Run a SCSI MMC command. 
  
-  cdio	        CD structure set by cdio_open().
+  cdio          CD structure set by cdio_open().
   i_timeout     time in milliseconds we will wait for the command
                 to complete. If this value is -1, use the default 
-		time-out value.
-  p_buf	        Buffer for data, both sending and receiving
-  i_buf	        Size of buffer
-  e_direction	direction the transfer is to go.
-  cdb	        CDB bytes. All values that are needed should be set on 
+                time-out value.
+  p_buf         Buffer for data, both sending and receiving
+  i_buf         Size of buffer
+  e_direction   direction the transfer is to go.
+  cdb           CDB bytes. All values that are needed should be set on 
                 input. We'll figure out what the right CDB length should be.
 
   We return true if command completed successfully and false if not.
  */
 static driver_return_code_t
-run_mmc_cmd_linux( void *p_user_data, 
-                   unsigned int i_timeout_ms,
-                   unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
-                   cdio_mmc_direction_t e_direction, 
-                   unsigned int i_buf, /*in/out*/ void *p_buf )
+run_mmc_cmd_linux(void *p_user_data, 
+                  unsigned int i_timeout_ms,
+                  unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
+                  cdio_mmc_direction_t e_direction, 
+                  unsigned int i_buf, /*in/out*/ void *p_buf)
 {
-  const _img_private_t *p_env = p_user_data;
+  _img_private_t *p_env = p_user_data;
   struct cdrom_generic_command cgc;
+  cdio_mmc_request_sense_t sense;
+  unsigned char *u_sense = (unsigned char *) &sense;
+
+  p_env->gen.scsi_mmc_sense_valid = 0;
   memset (&cgc, 0, sizeof (struct cdrom_generic_command));
   memcpy(&cgc.cmd, p_cdb, i_cdb);
   cgc.buflen = i_buf;
   cgc.buffer = p_buf;
-  cgc.data_direction = (SCSI_MMC_DATA_READ == cgc.data_direction)
-    ? CGC_DATA_READ : CGC_DATA_WRITE;
+  cgc.sense  = (struct request_sense *) &sense;
+
+  cgc.data_direction = (SCSI_MMC_DATA_READ  == e_direction) ? CGC_DATA_READ  :
+                       (SCSI_MMC_DATA_WRITE == e_direction) ? CGC_DATA_WRITE :
+                       CGC_DATA_NONE;
 
 #ifdef HAVE_LINUX_CDROM_TIMEOUT
   cgc.timeout = i_timeout_ms;
 #endif
 
+  memset(u_sense, 0, sizeof(sense));
   { 
     int i_rc = ioctl (p_env->gen.fd, CDROM_SEND_PACKET, &cgc);
+
+    /* Record SCSI sense reply for API call mmc_last_cmd_sense(). 
+    */
+    if (sense.additional_sense_len) {
+        /* SCSI Primary Command standard 
+           SPC 4.5.3, Table 26: 252 bytes legal, 263 bytes possible */
+        int sense_size = sense.additional_sense_len + 8; 
+    
+        if (sense_size > sizeof(sense))
+            sense_size = sizeof(sense);
+        memcpy((void *) p_env->gen.scsi_mmc_sense, &sense, sense_size);
+        p_env->gen.scsi_mmc_sense_valid = sense_size;
+    }
+
     if (0 == i_rc) return DRIVER_OP_SUCCESS;
     if (-1 == i_rc) {
-      cdio_info ("ioctl CDROM_SEND_PACKET failed: %s", strerror(errno));  
+      cdio_info("ioctl CDROM_SEND_PACKET for command %s (0x%0x) failed:\n\t%s", 
+                mmc_cmd2str((uint8_t) p_cdb->field[0]),
+                p_cdb->field[0], 
+                strerror(errno));  
       switch (errno) {
       case EPERM:
         return DRIVER_OP_NOT_PERMITTED;
@@ -1256,8 +1315,11 @@ run_mmc_cmd_linux( void *p_user_data,
         return DRIVER_OP_ERROR;
         break;
       }
-    }
-    return DRIVER_OP_ERROR;
+    } else if (i_rc < -1)
+      return DRIVER_OP_ERROR;
+    else
+      /*Not sure if this the best thing, but we'll use anyway. */
+      return DRIVER_OP_SUCCESS;
   }
 }
 
@@ -1408,7 +1470,7 @@ cdio_get_devices_linux (void)
       if (snprintf(drive, sizeof(drive), checklist2[i].format, j) < 0)
         continue;
       if ( (is_cdrom_linux(drive, NULL)) > 0 ) {
-	cdio_add_device_list(&drives, drive, &num_drives);
+        cdio_add_device_list(&drives, drive, &num_drives);
       }
     }
   }
@@ -1458,7 +1520,7 @@ cdio_get_default_device_linux(void)
       if (snprintf(drive, sizeof(drive), checklist2[i].format, j) < 0)
         continue;
       if ( is_cdrom_linux(drive, NULL) > 0 ) {
-	return(strdup(drive));
+        return(strdup(drive));
       }
     }
   }
@@ -1485,13 +1547,13 @@ close_tray_linux (const char *psz_device)
       switch(status) {
       case CDS_TRAY_OPEN:
         goto try_anyway;
-	break;
+        break;
       case CDS_DISC_OK:
-	cdio_info ("Tray already closed.");
+        cdio_info ("Tray already closed.");
         i_rc = DRIVER_OP_SUCCESS;
-	break;
+        break;
       default:
-	cdio_info ("Unknown CD-ROM status (%d), trying anyway", status);
+        cdio_info ("Unknown CD-ROM status (%d), trying anyway", status);
         goto try_anyway;
       }
     } else {
@@ -1513,6 +1575,79 @@ close_tray_linux (const char *psz_device)
 #endif /*HAVE_LINUX_CDROM*/
 }
 
+#ifdef HAVE_LINUX_CDROM
+/*!
+  Produce a text composed from the system SCSI address tuple according to
+  habits of Linux 2.4 and 2.6 :  "Bus,Host,Channel,Target,Lun" and store
+  it in generic_img_private_t.scsi_tuple.
+  To be accessed via cdio_get_arg("scsi-tuple-linux") or ("scsi-tuple").
+  Drivers which implement this code have to return 5 valid decimal numbers
+  separated by comma, or empty text if no such numbers are available.
+  @return   1=success , 0=failure
+*/
+static int
+set_scsi_tuple_linux(_img_private_t *env)
+{
+  int bus_no = -1, host_no = -1, channel_no = -1, target_no = -1, lun_no = -1; 
+  int ret, i;
+  char tuple[160], hdx[10];
+#ifdef SCSI_IOCTL_GET_IDLUN
+  struct my_scsi_idlun {
+    int x;
+    int host_unique_id;
+  };
+  struct my_scsi_idlun idlun;
+#endif
+  struct stat stbuf, env_stbuf;
+
+  /* Check whether this is a hdX and declare tuple unavailable.
+     /dev/hdX is traditionally for IDE drives and the ioctls here traditionally
+     return ok and all 0s for all IDE drives. So the tuples are no unique ids.
+  */
+  if (fstat(env->gen.fd, &env_stbuf) == -1)
+    goto no_tuple;
+  strcpy(hdx, "/dev/hdX");
+  for (i = 'a'; i <= 'z'; i++) {
+    hdx[7] = i;
+    if (stat(hdx, &stbuf) == -1)
+  continue;
+    if (env_stbuf.st_dev == stbuf.st_dev && env_stbuf.st_ino == stbuf.st_ino)
+      goto no_tuple;
+  }
+
+#ifdef SCSI_IOCTL_GET_BUS_NUMBER
+  if (ioctl(env->gen.fd, SCSI_IOCTL_GET_BUS_NUMBER, &bus_no) == -1)
+    bus_no = -1;
+#endif
+
+#ifdef SCSI_IOCTL_GET_IDLUN
+  /* http://www.tldp.org/HOWTO/SCSI-Generic-HOWTO/scsi_g_idlun.html */
+
+  ret = ioctl(env->gen.fd, SCSI_IOCTL_GET_IDLUN, &idlun);
+  if (ret != -1) {
+    host_no= (idlun.x >> 24) & 255;
+    channel_no= (idlun.x >> 16) & 255;
+    target_no= (idlun.x) & 255;
+    lun_no= (idlun.x >> 8) & 255;
+  }
+#endif
+
+  if (env->gen.scsi_tuple != NULL)
+    free (env->gen.scsi_tuple);
+  env->gen.scsi_tuple = NULL;
+  if (bus_no < 0 || host_no < 0 || channel_no < 0 || target_no < 0 ||
+      lun_no < 0) {
+no_tuple:;
+    env->gen.scsi_tuple = strdup("");
+    return 0;
+  }
+  snprintf(tuple, sizeof(tuple)-1, "%d,%d,%d,%d,%d",
+          bus_no, host_no, channel_no, target_no, lun_no);
+  env->gen.scsi_tuple = strdup(tuple);
+  return 1; 
+}
+#endif
+
 /*!
   Initialization routine. This is the only thing that doesn't
   get called via a function pointer. In fact *we* are the
@@ -1541,6 +1676,7 @@ cdio_open_am_linux (const char *psz_orig_source, const char *access_mode)
   CdIo_t *ret;
   _img_private_t *_data;
   char *psz_source;
+  int open_access_mode;  /* Access mode passed to cdio_generic_init. */
 
   cdio_funcs_t _funcs = {
     .audio_get_volume      = audio_get_volume_linux,
@@ -1560,6 +1696,7 @@ cdio_open_am_linux (const char *psz_orig_source, const char *access_mode)
     .get_arg               = get_arg_linux,
     .get_blocksize         = get_blocksize_mmc,
     .get_cdtext            = get_cdtext_generic,
+    .get_cdtext_raw        = read_cdtext_generic,
     .get_default_device    = cdio_get_default_device_linux,
     .get_devices           = cdio_get_devices_linux,
     .get_disc_last_lsn     = get_disc_last_lsn_linux,
@@ -1611,7 +1748,6 @@ cdio_open_am_linux (const char *psz_orig_source, const char *access_mode)
   _data->gen.init       = false;
   _data->gen.toc_init   = false;
   _data->gen.fd         = -1;
-  _data->gen.b_cdtext_init  = false;
   _data->gen.b_cdtext_error = false;
 
   if (NULL == psz_orig_source) {
@@ -1641,7 +1777,15 @@ cdio_open_am_linux (const char *psz_orig_source, const char *access_mode)
 
   ret->driver_id = DRIVER_LINUX;
 
-  if (cdio_generic_init(_data, O_RDONLY|O_NONBLOCK)) {
+  open_access_mode = O_NONBLOCK;
+  if (_AM_MMC_RDWR == _data->access_mode)
+    open_access_mode |= O_RDWR;
+  else if (_AM_MMC_RDWR_EXCL == _data->access_mode)
+    open_access_mode |= O_RDWR | O_EXCL;
+  else
+    open_access_mode |= O_RDONLY;
+  if (cdio_generic_init(_data, open_access_mode)) {
+    set_scsi_tuple_linux(_data);
     return ret;
   } else {
     cdio_generic_free (_data);
