@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003, 2004, 2005, 2006, 2008, 2009, 2011
+  Copyright (C) 2003, 2004, 2005, 2006, 2008, 2009, 2011, 2012
   Rocky Bernstein <rocky@gnu.org>
   Copyright (C) 2001, 2003 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -34,6 +34,12 @@
 #ifdef HAVE_GLOB_H
 #include <glob.h>
 #endif
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#endif
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif 
 
 #include <cdio/bytesex.h>
 #include <cdio/ds.h>
@@ -43,8 +49,7 @@
 #include "cdio_assert.h"
 #include "_cdio_stdio.h"
 #include "nrg.h"
-
-static const char _rcsid[] = "$Id: nrg.c,v 1.31 2008/04/21 18:30:22 karl Exp $";
+#include "cdtext_private.h"
 
 nero_id_t    nero_id;
 nero_dtype_t nero_dtype;
@@ -177,8 +182,8 @@ static bool
 parse_nrg (_img_private_t *p_env, const char *psz_nrg_name, 
 	   const cdio_log_level_t log_level)
 {
-  long unsigned int footer_start;
-  long unsigned int size;
+  off_t footer_start;
+  off_t size;
   char *footer_buf = NULL;
   if (!p_env) return false;
   size = cdio_stream_stat (p_env->gen.data_source);
@@ -208,11 +213,11 @@ parse_nrg (_img_private_t *p_env, const char *psz_nrg_name,
 
     cdio_assert ((size - footer_start) <= 4096);
 
-    footer_buf = calloc(1, size - footer_start);
+    footer_buf = calloc(1, (size_t)(size - footer_start));
 
     cdio_stream_seek (p_env->gen.data_source, footer_start, SEEK_SET);
     cdio_stream_read (p_env->gen.data_source, footer_buf, 
-		      size - footer_start, 1);
+		      (size_t)(size - footer_start), 1);
   }
   {
     int pos = 0;
@@ -431,7 +436,6 @@ parse_nrg (_img_private_t *p_env, const char *psz_nrg_name,
 	  }
 	  if (0 == disc_mode) {
 	    for (i=0; i<p_env->gen.i_tracks; i++) {
-	      cdtext_init (&(p_env->gen.cdtext_track[i]));
 	      p_env->tocent[i].track_format= track_format;
 	      p_env->tocent[i].datastart   = 0;
 	      p_env->tocent[i].track_green = false;
@@ -446,7 +450,6 @@ parse_nrg (_img_private_t *p_env, const char *psz_nrg_name,
 	    }
 	  } else if (2 == disc_mode) {
 	    for (i=0; i<p_env->gen.i_tracks; i++) {
-	      cdtext_init (&(p_env->gen.cdtext_track[i]));
 	      p_env->tocent[i].track_green = true;
 	      p_env->tocent[i].track_format= track_format;
 	      p_env->tocent[i].datasize    = CDIO_CD_FRAMESIZE;
@@ -748,8 +751,15 @@ parse_nrg (_img_private_t *p_env, const char *psz_nrg_name,
       case CDTX_ID: { /* "CD TEXT" */
         uint8_t *wdata = (uint8_t *) chunk->data;
         int len = UINT32_FROM_BE (chunk->len);
-        cdio_assert (len % sizeof (CDText_data_t) == 0);
-        cdtext_data_init (&p_env->gen, p_env->gen.i_first_track, &wdata[-4], len, set_cdtext_field_generic);
+        cdio_assert (len % CDTEXT_LEN_PACK == 0);
+        p_env->gen.cdtext = cdtext_init ();
+        if(0 !=cdtext_data_init (p_env->gen.cdtext, wdata, len))
+        {
+          cdtext_destroy(p_env->gen.cdtext);
+          free(p_env->gen.cdtext);
+          p_env->gen.cdtext = NULL;
+        }
+
         break;
       }
 
@@ -777,7 +787,6 @@ parse_nrg (_img_private_t *p_env, const char *psz_nrg_name,
   p_env->tocent[p_env->gen.i_tracks-1].sec_count = 
     cdio_lsn_to_lba(p_env->size - p_env->tocent[p_env->gen.i_tracks-1].start_lba);
 
-  p_env->gen.b_cdtext_init  = true;
   p_env->gen.b_cdtext_error = false;
   p_env->gen.toc_init       = true;
   free(footer_buf);
@@ -804,8 +813,6 @@ _init_nrg (_img_private_t *p_env)
   p_env->psz_mcn       = NULL;
   p_env->disc_mode     = CDIO_DISC_MODE_NO_INFO;
 
-  cdtext_init (&(p_env->gen.cdtext));
-
   if ( !parse_nrg (p_env, p_env->gen.source_name, CDIO_LOG_WARN) ) {
     cdio_warn ("image file %s is not a Nero image", 
 	       p_env->gen.source_name);
@@ -840,9 +847,9 @@ _lseek_nrg (void *p_user_data, off_t offset, int whence)
     track_info_t  *this_track=&(p_env->tocent[i]);
     p_env->pos.index = i;
     if ( (this_track->sec_count*this_track->datasize) >= offset) {
-      int blocks            = offset / this_track->datasize;
-      int rem               = offset % this_track->datasize;
-      int block_offset      = blocks * this_track->blocksize;
+      int blocks            = (int) (offset / this_track->datasize);
+      int rem               = (int) (offset % this_track->datasize);
+      off_t block_offset    = blocks * this_track->blocksize;
       real_offset          += block_offset + rem;
       p_env->pos.buff_offset = rem;
       p_env->pos.lba        += blocks;
@@ -1118,6 +1125,41 @@ _eject_media_nrg(void *obj)
   return DRIVER_OP_UNSUPPORTED;
 }
 
+#if !defined(HAVE_GLOB_H) && defined(_WIN32)
+static inline void Win32Glob(const char* pattern, const char* szCurPath, char ***drives, unsigned int *num_files)
+{
+  char szPath[MAX_PATH];
+  WIN32_FIND_DATAA ffd;
+  HANDLE hFind;
+  BOOL bFound;
+
+  SetCurrentDirectoryA(szCurPath);
+
+  hFind = FindFirstFileA(pattern, &ffd);
+  bFound = (hFind != INVALID_HANDLE_VALUE);
+  while (bFound) {
+    cdio_add_device_list(drives, ffd.cFileName, num_files);
+    bFound = FindNextFileA(hFind, &ffd);
+  }
+  if (hFind != INVALID_HANDLE_VALUE)
+    FindClose(hFind);
+
+  hFind = FindFirstFileA("*", &ffd);
+  bFound = (hFind != INVALID_HANDLE_VALUE);
+  while (bFound) {
+    if ( (ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
+         (strcmp(ffd.cFileName, ".") != 0) && (strcmp(ffd.cFileName, "..") != 0) ) {
+      GetFullPathNameA(ffd.cFileName, sizeof(szPath), szPath, NULL);
+      Win32Glob(pattern, szPath, drives, num_files);
+      SetCurrentDirectoryA(szCurPath);
+    }
+    bFound = FindNextFileA(hFind, &ffd);
+  }
+  if (hFind != INVALID_HANDLE_VALUE)
+    FindClose(hFind);
+}
+#endif
+
 /*!
   Return an array of strings giving possible NRG disk images.
  */
@@ -1135,6 +1177,10 @@ cdio_get_devices_nrg (void)
     cdio_add_device_list(&drives, globbuf.gl_pathv[i], &num_files);
   }
   globfree(&globbuf);
+#elif defined(_WIN32)
+  char szStartDir[MAX_PATH];
+  GetCurrentDirectoryA(sizeof(szStartDir), szStartDir);
+  Win32Glob("*.nrg", szStartDir, &drives, &num_files);
 #else
   cdio_add_device_list(&drives, DEFAULT_CDIO_DEVICE, &num_files);
 #endif /*HAVE_GLOB_H*/
@@ -1276,7 +1322,8 @@ cdio_open_nrg (const char *psz_source)
   _funcs.eject_media           = _eject_media_nrg;
   _funcs.free                  = _free_nrg;
   _funcs.get_arg               = _get_arg_image;
-  _funcs.get_cdtext            = get_cdtext_generic;
+  _funcs.get_cdtext            = _get_cdtext_image;
+  _funcs.get_cdtext_raw        = NULL;
   _funcs.get_devices           = cdio_get_devices_nrg;
   _funcs.get_default_device    = cdio_get_default_device_nrg;
   _funcs.get_disc_last_lsn     = get_disc_last_lsn_nrg;
