@@ -1,7 +1,5 @@
 /*
-  $Id: image_common.c,v 1.15 2008/04/22 15:29:12 karl Exp $
-
-  Copyright (C) 2004, 2005, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2004, 2005, 2008, 2010, 2011 Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -65,14 +63,13 @@ _free_image (void *p_user_data)
     track_info_t *p_tocent = &(p_env->tocent[i_track]);
     free_if_notnull(p_tocent->filename);
     free_if_notnull(p_tocent->isrc);
-    cdtext_destroy(&(p_tocent->cdtext));
     if (p_tocent->data_source) cdio_stdio_destroy(p_tocent->data_source);
   }
 
   free_if_notnull(p_env->psz_mcn);
   free_if_notnull(p_env->psz_cue_name);
   free_if_notnull(p_env->psz_access_mode);
-  cdtext_destroy(&(p_env->gen.cdtext));
+  cdtext_destroy(p_env->gen.cdtext);
   cdio_generic_stdio_free(p_env);
   free(p_env);
 }
@@ -91,10 +88,26 @@ _get_arg_image (void *user_data, const char key[])
     return p_env->psz_cue_name;
   } else if (!strcmp(key, "access-mode")) {
     return "image";
+  } else if (!strcmp (key, "mmc-supported?")) {
+    return "false";
   } 
   return NULL;
 }
 
+/*!
+  Return CD-Text object
+ */
+cdtext_t *
+_get_cdtext_image (void *user_data)
+{
+  generic_img_private_t *p_env = user_data;
+
+  if(!p_env)
+    return NULL;
+
+  return p_env->cdtext;
+}
+
 /*! 
   Get disc type associated with cd_obj.
 */
