@@ -1,5 +1,6 @@
 /*
   Copyright (C) 2009 KO Myung-Hun <komh@chollian.net>
+  Copyright (C) 2012 Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,15 +22,17 @@
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
-# define __CDIO_CONFIG_H__ 1
 #endif
 
-static const char _rcsid[] = "$Id: os2.c,v 1.30 2008/04/21 18:30:21 karl Exp $";
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif 
 
 #include <cdio/cdio.h>
 #include <cdio/sector.h>
 #include <cdio/util.h>
 #include <cdio/mmc.h>
+#include <cdio/logging.h>
 #include "cdio_assert.h"
 #include "cdio_private.h"
 
@@ -650,8 +653,6 @@ init_os2 (void *p_user_data)
 
   p_env->gen.init           = true;
   p_env->gen.toc_init       = false;
-  p_env->gen.b_cdtext_init  = false;
-  p_env->gen.b_cdtext_error = false;
   p_env->gen.fd             = p_env->h_cd;
 
   return true;
@@ -1447,6 +1448,7 @@ cdio_open_os2 (const char *psz_orig_source)
   _funcs.get_blocksize          = get_blocksize_os2;
 #endif
   _funcs.get_cdtext             = get_cdtext_generic;
+  _funcs.get_cdtext_raw         = read_cdtext_generic;
   _funcs.get_default_device     = cdio_get_default_device_os2;
   _funcs.get_devices            = cdio_get_devices_os2;
   _funcs.get_disc_last_lsn      = get_disc_last_lsn_os2;
@@ -1487,6 +1489,7 @@ cdio_open_os2 (const char *psz_orig_source)
 
   _data                 = calloc(1, sizeof (_img_private_t));
   _data->access_mode    = _AM_OS2;
+  _data->gen.b_cdtext_error = false;
   _data->gen.init       = false;
   _data->gen.fd         = -1;
 
@@ -1517,6 +1520,7 @@ cdio_open_os2 (const char *psz_orig_source)
     return ret;
   else {
     free_os2 (_data);
+    free(ret);
     return NULL;
   }
 #else
