@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003, 2004, 2005, 2006, 2008, 2010, 2011
+  Copyright (C) 2003, 2004, 2005, 2006, 2008, 2010, 2011, 2012
    Rocky Bernstein <rocky@gnu.org> 
   from vcdimager code: 
   Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
@@ -30,18 +30,32 @@
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
-# define __CDIO_CONFIG_H__ 1
 #endif
 
-static const char _rcsid[] = "$Id: osx.c,v 1.14 2008/10/17 11:58:52 rocky Exp $";
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif 
 
 #include <cdio/logging.h>
 #include <cdio/sector.h>
 #include <cdio/util.h>
 
 /* For SCSI TR_* enumerations */
-#include <cdio/cdda.h>
-
+typedef enum {
+  TR_OK =            0,
+  TR_EWRITE =        1  /**< Error writing packet command (transport) */,
+  TR_EREAD =         2  /**< Error reading packet data (transport) */,
+  TR_UNDERRUN =      3  /**< Read underrun */,
+  TR_OVERRUN =       4  /**< Read overrun */,
+  TR_ILLEGAL =       5  /**< Illegal/rejected request */,
+  TR_MEDIUM =        6  /**< Medium error */,
+  TR_BUSY =          7  /**< Device busy */,
+  TR_NOTREADY =      8  /**< Device not ready */,
+  TR_FAULT =         9  /**< Device failure */,
+  TR_UNKNOWN =      10  /**< Unspecified error */,
+  TR_STREAMING =    11  /**< loss of streaming */,
+} transport_error_t;
+  
 #include "cdio_assert.h"
 #include "cdio_private.h"
 
@@ -56,7 +70,11 @@ static const char _rcsid[] = "$Id: osx.c,v 1.14 2008/10/17 11:58:52 rocky Exp $"
 
 #include <mach/mach.h>
 #include <Carbon/Carbon.h>
-#include <IOKit/scsi-commands/SCSITaskLib.h>
+#if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 1030
+# include <IOKit/scsi/SCSITaskLib.h>
+#else
+# include <IOKit/scsi-commands/SCSITaskLib.h>
+#endif
 #include <IOKit/IOCFPlugIn.h>
 #include <mach/mach_error.h>
 
@@ -70,12 +88,15 @@ static const char _rcsid[] = "$Id: osx.c,v 1.14 2008/10/17 11:58:52 rocky Exp $"
 #include <sys/types.h>
 #include <sys/ioctl.h>
 
-
 #include <paths.h>
 #include <CoreFoundation/CoreFoundation.h>
 #include <IOKit/IOKitLib.h>
 #include <IOKit/IOBSD.h>
-#include <IOKit/scsi-commands/IOSCSIMultimediaCommandsDevice.h>
+#if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 1030
+# include <IOKit/scsi/IOSCSIMultimediaCommandsDevice.h>
+#else
+# include <IOKit/scsi-commands/IOSCSIMultimediaCommandsDevice.h>
+#endif
 #include <IOKit/storage/IOCDTypes.h>
 #include <IOKit/storage/IODVDTypes.h>
 #include <IOKit/storage/IOMedia.h>
@@ -85,6 +106,14 @@ static const char _rcsid[] = "$Id: osx.c,v 1.14 2008/10/17 11:58:52 rocky Exp $"
 #include <IOKit/storage/IODVDMediaBSDClient.h>
 #include <IOKit/storage/IOStorageDeviceCharacteristics.h>
 
+#if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 1050
+#include <IOKit/storage/IOBDTypes.h>
+#include <IOKit/storage/IOBDMedia.h>
+#include <IOKit/storage/IOBDMediaBSDClient.h>
+#else
+#define kIOBDMediaClass "IOBDMedia" // It does not hurt, simplyfies rest of code
+#endif
+
 #ifdef HAVE_DISKARBITRATION
 #include <DiskArbitration/DiskArbitration.h>
 #endif
@@ -157,7 +186,7 @@ GetRegistryEntryProperties ( io_service_t service )
   CFMutableDictionaryRef        dict    = 0;
   
   err = IORegistryEntryCreateCFProperties (service, &dict, 
-					   kCFAllocatorDefault, 0); 
+                                           kCFAllocatorDefault, 0); 
   if ( err != kIOReturnSuccess )
     cdio_warn( "IORegistryEntryCreateCFProperties: 0x%08x", err );
 
@@ -274,7 +303,7 @@ init_osx(_img_private_t *p_env) {
   
   ret = IOServiceGetMatchingServices( kIOMasterPortDefault, 
                                       IOBSDNameMatching(kIOMasterPortDefault, 
-							0, psz_devname),
+                                                        0, psz_devname),
                                       &iterator );
   
   /* Get service iterator for the device. */
@@ -288,10 +317,11 @@ init_osx(_img_private_t *p_env) {
   p_env->MediaClass_service = IOIteratorNext( iterator );
   IOObjectRelease( iterator );
 
-  /* search for kIOCDMediaClass or kIOCDVDMediaClass */ 
+  /* search for kIOCDMediaClass or kIODVDMediaClass or kIOBDMediaClass */ 
   while( p_env->MediaClass_service && 
          (!IOObjectConformsTo(p_env->MediaClass_service, kIOCDMediaClass)) &&
-         (!IOObjectConformsTo(p_env->MediaClass_service, kIODVDMediaClass)) )
+         (!IOObjectConformsTo(p_env->MediaClass_service, kIODVDMediaClass)) &&
+         (!IOObjectConformsTo(p_env->MediaClass_service, kIOBDMediaClass)) )
     {
 
       ret = IORegistryEntryGetParentIterator( p_env->MediaClass_service, 
@@ -305,12 +335,13 @@ init_osx(_img_private_t *p_env) {
         }
       
       IOObjectRelease( p_env->MediaClass_service );
+
       p_env->MediaClass_service = IOIteratorNext( iterator );
       IOObjectRelease( iterator );
     }
   
   if ( 0 == p_env->MediaClass_service )     {
-    cdio_warn( "search for kIOCDMediaClass/kIODVDMediaClass came up empty" );
+    cdio_warn( "search for kIOCDMediaClass/kIODVDMediaClass/kIOBDMediaClass came up empty" );
     return false;
   }
 
@@ -856,20 +887,6 @@ get_hwinfo_osx ( const CdIo_t *p_cdio, /*out*/ cdio_hwinfo_t *hw_info)
 }
 #endif
 
-/*
-  Get cdtext information in p_user_data for track i_track. 
-  For disc information i_track is 0.
-  
-  Return the CD-TEXT or NULL if obj is NULL,
-  CD-TEXT information does not exist, or (as is the case here)
-  we don't know how to get this implemented.
-*/
-static cdtext_t *
-get_cdtext_osx (void *p_user_data, track_t i_track) 
-{
-  return NULL;
-}
-
 static void 
 _free_osx (void *p_user_data) {
   _img_private_t *p_env = p_user_data;
@@ -1393,20 +1410,20 @@ static void media_eject_callback(DADiskRef disk, DADissenterRef dissenter, void
 
     if ( dissenter )
       {
-	CFStringRef status = DADissenterGetStatusString(dissenter);
-	if (status)
-	{ 
-		size_t cstr_size = CFStringGetLength(status);
-		char *cstr = malloc(cstr_size);
-		if ( CFStringGetCString( status,
-				 cstr, cstr_size,
-				 kCFStringEncodingASCII ) )
-	  	CFRelease( status );
+        CFStringRef status = DADissenterGetStatusString(dissenter);
+        if (status)
+        { 
+                size_t cstr_size = CFStringGetLength(status);
+                char *cstr = malloc(cstr_size);
+                if ( CFStringGetCString( status,
+                                 cstr, cstr_size,
+                                 kCFStringEncodingASCII ) )
+                CFRelease( status );
 
-		cdio_warn("%s", cstr);
+                cdio_warn("%s", cstr);
 
-		free(cstr);
-	}
+                free(cstr);
+        }
       }
 
     dacontext->result    = (dissenter ? DRIVER_OP_ERROR : DRIVER_OP_SUCCESS);
@@ -1470,27 +1487,27 @@ _eject_media_osx (void *user_data) {
   if ((disk = DADiskCreateFromBSDName(kCFAllocatorDefault, dacontext.session, psz_drive)) != NULL)
     {
       if ((description = DADiskCopyDescription(disk)) != NULL)
-	{
-	  /* Does the device need to be unmounted first? */
-	  DASessionScheduleWithRunLoop(dacontext.session, dacontext.runloop, kCFRunLoopDefaultMode);
-	  CFRunLoopAddSource(dacontext.runloop, dacontext.cancel, kCFRunLoopDefaultMode);
+        {
+          /* Does the device need to be unmounted first? */
+          DASessionScheduleWithRunLoop(dacontext.session, dacontext.runloop, kCFRunLoopDefaultMode);
+          CFRunLoopAddSource(dacontext.runloop, dacontext.cancel, kCFRunLoopDefaultMode);
 
-	  if (CFDictionaryGetValueIfPresent(description, kDADiskDescriptionVolumePathKey, NULL))
-	    {
-	      DADiskUnmount(disk, kDADiskUnmountOptionDefault, media_unmount_callback, &dacontext);
+          if (CFDictionaryGetValueIfPresent(description, kDADiskDescriptionVolumePathKey, NULL))
+            {
+              DADiskUnmount(disk, kDADiskUnmountOptionDefault, media_unmount_callback, &dacontext);
             }
-	  else
-	    {
-	      DADiskEject(disk, kDADiskEjectOptionDefault, media_eject_callback, &dacontext);
-	      dacontext.result = dacontext.result == DRIVER_OP_UNINIT ? DRIVER_OP_SUCCESS : dacontext.result;
+          else
+            {
+              DADiskEject(disk, kDADiskEjectOptionDefault, media_eject_callback, &dacontext);
+              dacontext.result = dacontext.result == DRIVER_OP_UNINIT ? DRIVER_OP_SUCCESS : dacontext.result;
             }
-	  if (!dacontext.completed)
-	    {
-	      CFRunLoopRunInMode(kCFRunLoopDefaultMode, 30.0, TRUE);  /* timeout after 30 seconds */
+          if (!dacontext.completed)
+            {
+              CFRunLoopRunInMode(kCFRunLoopDefaultMode, 30.0, TRUE);  /* timeout after 30 seconds */
             }
-	  CFRunLoopRemoveSource(dacontext.runloop, dacontext.cancel, kCFRunLoopDefaultMode);
-	  DASessionUnscheduleFromRunLoop(dacontext.session, dacontext.runloop, kCFRunLoopDefaultMode);
-	  CFRelease(description);
+          CFRunLoopRemoveSource(dacontext.runloop, dacontext.cancel, kCFRunLoopDefaultMode);
+          DASessionUnscheduleFromRunLoop(dacontext.session, dacontext.runloop, kCFRunLoopDefaultMode);
+          CFRelease(description);
         }
       CFRelease(disk);
     }
@@ -1826,6 +1843,12 @@ cdio_get_default_device_osx(void)
       
       do
         {
+          /* Skip other removable media, like USB flash memory keys:  */
+          if (!IOObjectConformsTo(next_media, kIODVDMediaClass) &&
+              !IOObjectConformsTo(next_media, kIOCDMediaClass) &&
+              !IOObjectConformsTo(next_media, kIOBDMediaClass))
+            continue;
+
           str_bsd_path = IORegistryEntryCreateCFProperty( next_media,
                                                           CFSTR( kIOBSDNameKey ),
                                                           kCFAllocatorDefault,
@@ -1856,6 +1879,7 @@ cdio_get_default_device_osx(void)
         } while( ( next_media = IOIteratorNext( media_iterator ) ) != 0 );
     }
   IOObjectRelease( media_iterator );
+  cdio_warn ("cdio_get_default_device() - No CD/DVD/BD media - returning NULL");
   return NULL;
 #endif /* HAVE_DARWIN_CDROM */
 }
@@ -1893,7 +1917,8 @@ cdio_open_osx (const char *psz_orig_source)
     .eject_media           = _eject_media_osx,
     .free                  = _free_osx,
     .get_arg               = _get_arg_osx,
-    .get_cdtext            = get_cdtext_osx,
+    .get_cdtext            = get_cdtext_generic,
+    .get_cdtext_raw        = read_cdtext_generic,
     .get_default_device    = cdio_get_default_device_osx,
     .get_devices           = cdio_get_devices_osx,
     .get_disc_last_lsn     = get_disc_last_lsn_osx,
@@ -1930,15 +1955,15 @@ cdio_open_osx (const char *psz_orig_source)
   _data->gen.init           = false;
   _data->gen.fd             = -1;
   _data->gen.toc_init       = false;
-  _data->gen.b_cdtext_init  = false;
   _data->gen.b_cdtext_error = false;
 
   if (NULL == psz_orig_source) {
     psz_source=cdio_get_default_device_osx();
     if (NULL == psz_source) { 
- 	cdio_generic_free(_data); 
- 	return NULL; 
+        free(_data);
+        return NULL; 
     } 
+
     _set_arg_osx(_data, "source", psz_source);
     free(psz_source);
   } else {
@@ -1949,21 +1974,21 @@ cdio_open_osx (const char *psz_orig_source)
 #if 0
       cdio_info ("source %s is a not a device", psz_orig_source);
 #endif
-      cdio_generic_free(_data); 
+      free(_data);
       return NULL;
     }
   }
 
   ret = cdio_new ((void *)_data, &_funcs);
-  if (ret == NULL) {
-    cdio_generic_free(_data);
-    return NULL; 
-  } 
+  if (ret == NULL) return NULL;
+
   ret->driver_id = DRIVER_OSX;
+
   if (cdio_generic_init(_data, O_RDONLY | O_NONBLOCK) && init_osx(_data))
     return ret;
   else {
     cdio_generic_free (_data);
+    free(ret);
     return NULL;
   }
   
