@@ -1,7 +1,5 @@
 /*
-    $Id: sector.c,v 1.5 2005/02/06 04:20:25 rocky Exp $
-
-    Copyright (C) 2004, 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2004, 2005, 2011, 2012 Rocky Bernstein <rocky@gnu.org>
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
     This program is free software; you can redistribute it and/or modify
@@ -21,14 +19,18 @@
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
+# define __CDIO_CONFIG_H__ 1
 #endif
 
 #include <cdio/sector.h>
 #include <cdio/util.h>
 #include <cdio/logging.h>
 #include "cdio_assert.h"
+#include "portable.h"
 
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -193,7 +195,7 @@ cdio_mmssff_to_lba (const char *psz_mmssff)
 {
   int psz_field;
   lba_t ret;
-  char c;
+  unsigned char c;
   
   if (0 == strcmp (psz_mmssff, "0"))
     return 0;
