@@ -1,22 +1,20 @@
 /*
-    $Id: iso9660.c,v 1.34 2007/11/16 22:50:21 flameeyes Exp $
+  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+  Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
-    Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003, 2004, 2005, 2006, 2007 Rocky Bernstein <rocky@gnu.org>
+  This program is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /*! String inside frame which identifies an ISO 9660 filesystem. This
@@ -38,26 +36,90 @@ const char ISO_STANDARD_ID[] = {'C', 'D', '0', '0', '1'};
 #include <time.h>
 #include <ctype.h>
 #ifdef HAVE_STRING_H
-#include <string.h>
+# include <string.h>
 #endif
 #ifdef HAVE_STDIO_H
-#include <stdio.h>
+# include <stdio.h>
 #endif
 #ifdef HAVE_LIMITS_H
-#include <limits.h>
+# include <limits.h>
 #endif
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h>
+# include <stdlib.h>
 #endif
 #ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
+# include <sys/stat.h>
 #endif
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
 
-static const char _rcsid[] = "$Id: iso9660.c,v 1.34 2007/11/16 22:50:21 flameeyes Exp $";
+#ifndef HAVE_SETENV
+static int
+setenv(const char *envname, const char *envval, int overwrite)
+{
+  return -1;
+}
+#endif
+
+#ifndef HAVE_UNSETENV
+static int
+unsetenv(const char *envname)
+{
+  return -1;
+}
+#endif
+
+#ifndef HAVE_TIMEGM
+static time_t
+timegm(struct tm *tm)
+{
+  time_t ret;
+  char *tz;
+  
+  tz = getenv("TZ");
+  setenv("TZ", "UTC", 1);
+  tzset();
+  ret = mktime(tm);
+  if (tz)
+    setenv("TZ", tz, 1);
+  else
+    unsetenv("TZ");
+  tzset();
+  return ret;
+}
+#endif
+
+#ifndef HAVE_GMTIME_R
+static struct tm *
+gmtime_r(const time_t *timer, struct tm *result)
+{
+    struct tm *tmp = gmtime(timer);
+
+    if (tmp) {
+        *result = *tmp;
+        return result;
+    }
+    return tmp;
+}
+#endif
+
+#ifndef HAVE_LOCALTIME_R
+static struct tm *
+localtime_r(const time_t *timer, struct tm *result)
+{
+    struct tm *tmp = localtime(timer);
+
+    if (tmp) {
+        *result = *tmp;
+        return result;
+    }
+    return tmp;
+}
+#endif
+
+static const char _rcsid[] = "$Id: iso9660.c,v 1.41 2008/06/25 08:01:54 rocky Exp $";
 
 /* Variables to hold debugger-helping enumerations */
 enum iso_enum1_s     iso_enums1;
@@ -128,10 +190,10 @@ iso9660_get_dtime (const iso9660_dtime_t *idr_date, bool b_localtime,
        0 == idr_date->dt_day    && 0 == idr_date->dt_hour  &&
        0 == idr_date->dt_minute && 0 == idr_date->dt_second ) {
     time_t t = 0;
-    struct tm *p_temp_tm;
-    p_temp_tm = localtime(&t);
+    struct tm temp_tm;
+    localtime_r(&t, &temp_tm);
     
-    memcpy(p_tm, p_temp_tm, sizeof(struct tm));
+    memcpy(p_tm, &temp_tm, sizeof(struct tm));
     return true;
   }
   
@@ -149,18 +211,16 @@ iso9660_get_dtime (const iso9660_dtime_t *idr_date, bool b_localtime,
      date values to account for the timezone offset. */
   {
     time_t t = 0;
-    struct tm *p_temp_tm = b_localtime ? localtime(&t): gmtime(&t);
-    
-    t = mktime(p_tm);
-    
-    p_temp_tm = b_localtime ? localtime(&t) : gmtime(&t);
+    struct tm temp_tm;
 
-    if (p_temp_tm->tm_isdst) {
-      t += 60*60;
-      p_temp_tm = b_localtime ? localtime(&t) : gmtime(&t);
-    }
+    t = timegm(p_tm);
     
-    memcpy(p_tm, p_temp_tm, sizeof(struct tm));
+    if (b_localtime)
+      localtime_r(&t, &temp_tm);
+    else
+      gmtime_r(&t, &temp_tm);
+
+    memcpy(p_tm, &temp_tm, sizeof(struct tm));
   }
 
 
@@ -211,37 +271,38 @@ iso9660_get_ltime (const iso9660_ltime_t *p_ldate,
   set_ltime_field(tm_min,  lt_minute, 0);
   set_ltime_field(tm_sec,  lt_second, 0);
   p_tm->tm_isdst= -1; /* information not available */
+#ifndef HAVE_TM_GMTOFF
   p_tm->tm_sec += p_ldate->lt_gmtoff * (15 * 60);
+#endif
 
   /* Recompute tm_wday and tm_yday via mktime. mktime will also renormalize
      date values to account for the timezone offset. */
   {
     time_t t;
-    struct tm *p_temp_tm;
-    const char *old_tzname=getenv("TZ");
-    char psz_gmt_tzset[]="TZ=GMT";
+    struct tm temp_tm;
 
-    putenv(psz_gmt_tzset);
-    tzset();
     t = mktime(p_tm);
-    p_temp_tm = gmtime(&t);
-    if (old_tzname) {
-      char psz_tzset[10];
-      snprintf(psz_tzset, sizeof(psz_tzset), "TZ=%s", old_tzname);
-    }
-    
-    p_tm->tm_wday = p_temp_tm->tm_wday;
-    p_tm->tm_yday = p_temp_tm->tm_yday;
+
+    localtime_r(&t, &temp_tm);
+
+    memcpy(p_tm, &temp_tm, sizeof(struct tm));
   }
+  p_tm->tm_isdst= -1; /* information not available */
+#ifdef HAVE_TM_GMTOFF
+  p_tm->tm_gmtoff = -p_ldate->lt_gmtoff * (15 * 60);
+#endif
   return true;
 }
 
-
 /*!
   Set time in format used in ISO 9660 directory index record
-  from a Unix time structure. */
+  from a Unix time structure. timezone is given as an offset
+  correction in minutes. 
+*/
 void
-iso9660_set_dtime (const struct tm *p_tm, /*out*/ iso9660_dtime_t *p_idr_date)
+iso9660_set_dtime_with_timezone (const struct tm *p_tm, 
+                                 int time_zone,
+                                 /*out*/ iso9660_dtime_t *p_idr_date)
 {
   memset (p_idr_date, 0, 7);
 
@@ -254,12 +315,9 @@ iso9660_set_dtime (const struct tm *p_tm, /*out*/ iso9660_dtime_t *p_idr_date)
   p_idr_date->dt_minute = p_tm->tm_min;
   p_idr_date->dt_second = p_tm->tm_sec;
 
-#ifdef HAVE_TM_GMTOFF
   /* The ISO 9660 timezone is in the range -48..+52 and each unit
      represents a 15-minute interval. */
-  p_idr_date->dt_gmtoff = p_tm->tm_gmtoff / (15 * 60);
-
-  if (p_tm->tm_isdst > 0) p_idr_date->dt_gmtoff -= 4;
+  p_idr_date->dt_gmtoff = time_zone / 15;
 
   if (p_idr_date->dt_gmtoff < -48 ) {
     
@@ -271,21 +329,40 @@ iso9660_set_dtime (const struct tm *p_tm, /*out*/ iso9660_dtime_t *p_idr_date)
                p_idr_date->dt_gmtoff);
     p_idr_date->dt_gmtoff = 52;
   }
+}
+
+/*!
+  Set time in format used in ISO 9660 directory index record
+  from a Unix time structure. */
+void
+iso9660_set_dtime(const struct tm *p_tm, /*out*/ iso9660_dtime_t *p_idr_date)
+{
+  int time_zone = 0;
+  if (p_tm) {
+#ifdef HAVE_TM_GMTOFF
+    /* Convert seconds to minutes */
+    time_zone = p_tm->tm_gmtoff / 60;
 #else 
-  p_idr_date->dt_gmtoff = 0;
+    time_zone = (p_tm->tm_isdst > 0) ? -60 : 0;
 #endif
+  }
+  iso9660_set_dtime_with_timezone (p_tm, time_zone, p_idr_date);
 }
 
 /*!
   Set "long" time in format used in ISO 9660 primary volume descriptor
-  from a Unix time structure. */
+  from a Unix time structure. timezone is given as an offset
+  correction in minutes. 
+*/
 void
-iso9660_set_ltime (const struct tm *p_tm, /*out*/ iso9660_ltime_t *pvd_date)
+iso9660_set_ltime_with_timezone(const struct tm *p_tm, 
+                                int time_zone,
+                                /*out*/ iso9660_ltime_t *pvd_date)
 {
   char *_pvd_date = (char *) pvd_date; 
 
   memset (_pvd_date, '0', 16);
-  _pvd_date[16] = (iso712_t) 0; /* Start out with time zone GMT. */
+  pvd_date->lt_gmtoff = (iso712_t) 0; /* Start out with time zone GMT. */
 
   if (!p_tm) return;
 
@@ -295,24 +372,36 @@ iso9660_set_ltime (const struct tm *p_tm, /*out*/ iso9660_ltime_t *pvd_date)
            p_tm->tm_hour, p_tm->tm_min, p_tm->tm_sec,
            0 /* 1/100 secs */ );
 
-  /* Adjust for daylight savings time. */
-  if (p_tm->tm_isdst > 0) _pvd_date[16] -= (iso712_t) 4;
-
-#ifdef HAVE_TM_GMTOFF
   /* Set time zone in 15-minute interval encoding. */
-  _pvd_date[16] -= p_tm->tm_gmtoff / (15 * 60);
-  if (_pvd_date[16] < -48 ) {
+  pvd_date->lt_gmtoff -= (time_zone / 15);
+  if (pvd_date->lt_gmtoff < -48 ) {
     
     cdio_warn ("Converted ISO 9660 timezone %d is less than -48. Adjusted", 
-               (int) _pvd_date[16]);
-    _pvd_date[16] = -48;
-  } else if (_pvd_date[16] > 52) {
+               (int) pvd_date->lt_gmtoff);
+    pvd_date->lt_gmtoff = -48;
+  } else if (pvd_date->lt_gmtoff > 52) {
     cdio_warn ("Converted ISO 9660 timezone %d is over 52. Adjusted", 
-               (int) _pvd_date[16]);
-    _pvd_date[16] = 52;
+               (int) pvd_date->lt_gmtoff);
+    pvd_date->lt_gmtoff = 52;
   }
-#endif
+}
 
+/*!
+  Set "long" time in format used in ISO 9660 primary volume descriptor
+  from a Unix time structure. */
+void
+iso9660_set_ltime(const struct tm *p_tm, /*out*/ iso9660_ltime_t *pvd_date)
+{
+  int time_zone = 0;
+  if (p_tm) {
+#ifdef HAVE_TM_GMTOFF
+    /* Set time zone in 15-minute interval encoding. */
+    time_zone = p_tm->tm_gmtoff / 60;
+#else
+    time_zone = (p_tm->tm_isdst > 0) ? -60 : 0;
+#endif
+  }
+  iso9660_set_ltime_with_timezone (p_tm, time_zone, pvd_date);
 }
 
 /*!
@@ -525,6 +614,7 @@ iso9660_set_pvd(void *pd,
                 )
 {
   iso9660_pvd_t ipd;
+  struct tm temp_tm;
 
   cdio_assert (sizeof(iso9660_pvd_t) == ISO_BLOCKSIZE);
 
@@ -576,10 +666,12 @@ iso9660_set_pvd(void *pd,
   iso9660_strncpy_pad (ipd.abstract_file_id     , "", 37, ISO9660_DCHARS);
   iso9660_strncpy_pad (ipd.bibliographic_file_id, "", 37, ISO9660_DCHARS);
 
-  iso9660_set_ltime (gmtime (pvd_time), &(ipd.creation_date));
-  iso9660_set_ltime (gmtime (pvd_time), &(ipd.modification_date));
-  iso9660_set_ltime (NULL,              &(ipd.expiration_date));
-  iso9660_set_ltime (NULL,              &(ipd.effective_date));
+  gmtime_r(pvd_time, &temp_tm);
+  iso9660_set_ltime (&temp_tm, &(ipd.creation_date));
+  gmtime_r(pvd_time, &temp_tm);
+  iso9660_set_ltime (&temp_tm, &(ipd.modification_date));
+  iso9660_set_ltime (NULL,     &(ipd.expiration_date));
+  iso9660_set_ltime (NULL,     &(ipd.effective_date));
 
   ipd.file_structure_version = to_711(1);
 
@@ -619,6 +711,7 @@ iso9660_dir_add_entry_su(void *dir,
   unsigned int offset = 0;
   uint32_t dsize = from_733(idr->size);
   int length, su_offset;
+  struct tm temp_tm;
   cdio_assert (sizeof(iso9660_dir_t) == 33);
 
   if (!dsize && !idr->length)
@@ -675,16 +768,17 @@ iso9660_dir_add_entry_su(void *dir,
   idr->extent = to_733(extent);
   idr->size = to_733(size);
   
-  iso9660_set_dtime (gmtime(entry_time), &(idr->recording_time));
+  gmtime_r(entry_time, &temp_tm);
+  iso9660_set_dtime (&temp_tm, &(idr->recording_time));
   
   idr->file_flags = to_711(file_flags);
 
   idr->volume_sequence_number = to_723(1);
 
-  idr->filename_len = to_711(strlen(filename) 
+  idr->filename.len = to_711(strlen(filename) 
                              ? strlen(filename) : 1); /* working hack! */
 
-  memcpy(idr->filename, filename, from_711(idr->filename_len));
+  memcpy(&idr->filename.str[1], filename, from_711(idr->filename.len));
   memcpy(&dir8[offset] + su_offset, su_data, su_size);
 }
 
@@ -745,11 +839,14 @@ iso9660_get_posix_filemode(const iso9660_stat_t *p_iso_dirent)
 {
   mode_t mode = 0;
 
-  if (yep == p_iso_dirent->rr.b3_rock) {
-    return iso9660_get_posix_filemode_from_rock(&p_iso_dirent->rr);
-  } else if (p_iso_dirent->b_xa) {
-    return iso9660_get_posix_filemode_from_xa(p_iso_dirent->xa.attributes);
-  }
+#ifdef HAVE_ROCK 
+  if (yep == p_iso_dirent->rr.b3_rock) { 
+      return iso9660_get_posix_filemode_from_rock(&p_iso_dirent->rr); 
+  } else 
+#endif 
+  if (p_iso_dirent->b_xa) { 
+    return iso9660_get_posix_filemode_from_xa(p_iso_dirent->xa.attributes); 
+  } 
   return mode;
 }
 
@@ -1033,7 +1130,7 @@ iso9660_get_application_id(iso9660_pvd_t *p_pvd)
   return strdup(strip_trail(p_pvd->application_id, ISO_MAX_APPLICATION_ID));
 }
 
-#if FIXME
+#ifdef FIXME
 lsn_t
 iso9660_get_dir_extent(const iso9660_dir_t *idr) 
 {
@@ -1049,7 +1146,7 @@ iso9660_get_dir_len(const iso9660_dir_t *idr)
   return idr->length;
 }
 
-#if FIXME
+#ifdef FIXME
 uint8_t
 iso9660_get_dir_size(const iso9660_dir_t *idr) 
 {
