@@ -1,37 +1,42 @@
 /*
-    $Id: iso9660_fs.c,v 1.46 2008/02/08 08:53:33 rocky Exp $
-
-    Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 
+  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2011, 2012
     Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is free software: you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-    02110-1301 USA.
+  You should have received a copy of the GNU General Public License
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /* iso9660 filesystem-based routines */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
+#if defined(HAVE_CONFIG_H) && !defined(__CDIO_CONFIG_H__)
+#include "config.h"
+#define __CDIO_CONFIG_H__ 1
+#endif
+
+#ifdef HAVE_STDBOOL_H
+# include <stdbool.h>
+#endif 
+
+#ifdef HAVE_STDIO_H
+#include <stdio.h>
 #endif
 
 #ifdef HAVE_STRING_H
-# include <string.h>
+#include <string.h>
 #endif
 
 #ifdef HAVE_ERRNO_H
-# include <errno.h>
+#include <errno.h>
 #endif
 
 #ifdef HAVE_LANGINFO_CODESET
@@ -49,9 +54,7 @@
 #include "_cdio_stdio.h"
 #include "cdio_private.h"
 
-#include <stdio.h>
-
-static const char _rcsid[] = "$Id: iso9660_fs.c,v 1.46 2008/02/08 08:53:33 rocky Exp $";
+static const char _rcsid[] = "$Id: iso9660_fs.c,v 1.47 2008/04/18 16:02:09 karl Exp $";
 
 /* Implementation of iso9660_t type */
 struct _iso9660_s {
@@ -110,12 +113,10 @@ adjust_fuzzy_pvd( iso9660_t *p_iso )
      frame and a header. 
    */
   if (CDIO_CD_FRAMESIZE_RAW == p_iso->i_framesize) {
-    const int pre_user_data=CDIO_CD_SYNC_SIZE +  CDIO_CD_HEADER_SIZE 
-      + CDIO_CD_SUBHEADER_SIZE;
-    char buf[pre_user_data];
-    
-    i_byte_offset -= pre_user_data;
-    
+    char buf[CDIO_CD_SYNC_SIZE + CDIO_CD_HEADER_SIZE + CDIO_CD_SUBHEADER_SIZE];
+
+    i_byte_offset -= CDIO_CD_SYNC_SIZE + CDIO_CD_HEADER_SIZE + CDIO_CD_SUBHEADER_SIZE;
+
     if ( DRIVER_OP_SUCCESS != cdio_stream_seek (p_iso->stream, i_byte_offset, 
 						SEEK_SET) )
       return;
@@ -194,10 +195,9 @@ iso9660_open_ext_private (const char *psz_path,
   return p_iso;
 
  error:
-  if (p_iso && p_iso->stream) {
-    cdio_stdio_destroy(p_iso->stream);
-    free(p_iso);
-  }
+  if (p_iso && p_iso->stream) cdio_stdio_destroy(p_iso->stream);
+  free(p_iso);
+  
   return NULL;
 }
 
@@ -278,6 +278,72 @@ check_pvd (const iso9660_pvd_t *p_pvd, cdio_log_level_t log_level)
   return true;
 }
 
+
+/*!
+  Core procedure for the iso9660_ifs_get_###_id() calls.
+  pvd_member/svd_member is a pointer to an achar_t or dchar_t
+  ID string which we can superset as char.
+  If the Joliet converted string is the same as the achar_t/dchar_t
+  one, we fall back to using the latter, as it may be longer.
+*/
+static inline bool
+get_member_id(iso9660_t *p_iso, cdio_utf8_t **p_psz_member_id,
+              char* pvd_member, char* svd_member, size_t max_size)
+{
+  int j;
+  bool strip;
+
+  if (!p_iso) {
+    *p_psz_member_id = NULL;
+    return false;
+  }
+#ifdef HAVE_JOLIET  
+  if (p_iso->i_joliet_level) {
+    /* Translate USC-2 string from Secondary Volume Descriptor */
+    if (cdio_charset_to_utf8(svd_member, max_size,
+                            p_psz_member_id, "UCS-2BE")) {
+      /* NB: *p_psz_member_id is never NULL on success. */
+      if (strncmp(*p_psz_member_id, pvd_member,
+                  strlen(*p_psz_member_id)) != 0) {
+        /* Strip trailing spaces */
+        for (j = strlen(*p_psz_member_id)-1; j >= 0; j--) {
+          if ((*p_psz_member_id)[j] != ' ')
+            break;
+          (*p_psz_member_id)[j] = '\0';
+        }
+        if ((*p_psz_member_id)[0] != 0) {
+          /* Joliet string is not empty and differs from
+             non Joliet one => use it */
+          return true;
+        }
+      }
+      /* Joliet string was either empty or same */
+      free(*p_psz_member_id);
+    }
+  }
+#endif /*HAVE_JOLIET*/
+  *p_psz_member_id = calloc(max_size+1, sizeof(cdio_utf8_t));
+  if (!*p_psz_member_id) {
+    cdio_warn("Memory allocation error");
+    return false;
+  }
+  /* Copy string while removing trailing spaces */
+  (*p_psz_member_id)[max_size] = 0;
+  for (strip=true, j=max_size-1; j>=0; j--) {
+    if (strip && (pvd_member[j] == ' '))
+      continue;
+    strip = false;
+    (*p_psz_member_id)[j] = pvd_member[j];
+  }
+  if (strlen(*p_psz_member_id) == 0) {
+    free(*p_psz_member_id);
+    *p_psz_member_id = NULL;
+    return false;
+  }
+  return true;
+}
+
+
 /*!  
   Return the application ID.  NULL is returned in psz_app_id if there
   is some problem in getting this.
@@ -286,30 +352,14 @@ bool
 iso9660_ifs_get_application_id(iso9660_t *p_iso, 
 			       /*out*/ cdio_utf8_t **p_psz_app_id)
 {
-  if (!p_iso) {
-    *p_psz_app_id = NULL;
-    return false;
-  }
-
-#ifdef HAVE_JOLIET  
-  if (p_iso->i_joliet_level) {
-    /* TODO: check that we haven't reached the maximum size.
-       If we have, perhaps we've truncated and if we can get 
-       longer results *and* have the same character using
-       the PVD, do that.
-     */
-  if ( cdio_charset_to_utf8(p_iso->svd.application_id,
-                            ISO_MAX_APPLICATION_ID,
-                            p_psz_app_id, "UCS-2BE"))
-      return true;
-  }
-#endif /*HAVE_JOLIET*/ 
-  *p_psz_app_id = iso9660_get_application_id( &(p_iso->pvd) );
-  return *p_psz_app_id != NULL && strlen(*p_psz_app_id);
+  return get_member_id(p_iso, p_psz_app_id,
+                       (char*)p_iso->pvd.application_id,
+                       (char*)p_iso->svd.application_id,
+                       ISO_MAX_APPLICATION_ID);
 }
 
 /*!  
-  Return the Joliet level recognaized for p_iso.
+  Return the Joliet level recognized for p_iso.
 */
 uint8_t iso9660_ifs_get_joliet_level(iso9660_t *p_iso)
 {
@@ -325,25 +375,10 @@ bool
 iso9660_ifs_get_preparer_id(iso9660_t *p_iso,
 			/*out*/ cdio_utf8_t **p_psz_preparer_id)
 {
-  if (!p_iso) {
-    *p_psz_preparer_id = NULL;
-    return false;
-  }
-
-#ifdef HAVE_JOLIET  
-  if (p_iso->i_joliet_level) {
-    /* TODO: check that we haven't reached the maximum size.
-       If we have, perhaps we've truncated and if we can get 
-       longer results *and* have the same character using
-       the PVD, do that.
-     */
-    if ( cdio_charset_to_utf8(p_iso->svd.preparer_id, ISO_MAX_PREPARER_ID,
-                              p_psz_preparer_id, "UCS-2BE") )
-      return true;
-  }
-#endif /*HAVE_JOLIET*/
-  *p_psz_preparer_id = iso9660_get_preparer_id( &(p_iso->pvd) );
-  return *p_psz_preparer_id != NULL && strlen(*p_psz_preparer_id);
+  return get_member_id(p_iso, p_psz_preparer_id,
+                       (char*)p_iso->pvd.preparer_id,
+                       (char*)p_iso->svd.preparer_id,
+                       ISO_MAX_PREPARER_ID);
 }
 
 /*!
@@ -353,28 +388,12 @@ iso9660_ifs_get_preparer_id(iso9660_t *p_iso,
 bool iso9660_ifs_get_publisher_id(iso9660_t *p_iso,
                                   /*out*/ cdio_utf8_t **p_psz_publisher_id)
 {
-  if (!p_iso) {
-    *p_psz_publisher_id = NULL;
-    return false;
-  }
-
-#ifdef HAVE_JOLIET  
-  if (p_iso->i_joliet_level) {
-    /* TODO: check that we haven't reached the maximum size.
-       If we have, perhaps we've truncated and if we can get 
-       longer results *and* have the same character using
-       the PVD, do that.
-     */
-    if( cdio_charset_to_utf8(p_iso->svd.publisher_id, ISO_MAX_PUBLISHER_ID,
-                             p_psz_publisher_id, "UCS-2BE") )
-      return true;
-  }
-#endif /*HAVE_JOLIET*/
-  *p_psz_publisher_id = iso9660_get_publisher_id( &(p_iso->pvd) );
-  return *p_psz_publisher_id != NULL && strlen(*p_psz_publisher_id);
+  return get_member_id(p_iso, p_psz_publisher_id,
+                       (char*)p_iso->pvd.publisher_id,
+                       (char*)p_iso->svd.publisher_id,
+                       ISO_MAX_PUBLISHER_ID);
 }
 
-
 /*!
    Return a string containing the PVD's publisher id with trailing
    blanks removed.
@@ -382,28 +401,12 @@ bool iso9660_ifs_get_publisher_id(iso9660_t *p_iso,
 bool iso9660_ifs_get_system_id(iso9660_t *p_iso,
 			       /*out*/ cdio_utf8_t **p_psz_system_id)
 {
-  if (!p_iso) {
-    *p_psz_system_id = NULL;
-    return false;
-  }
-
-#ifdef HAVE_JOLIET  
-  if (p_iso->i_joliet_level) {
-    /* TODO: check that we haven't reached the maximum size.
-       If we have, perhaps we've truncated and if we can get 
-       longer results *and* have the same character using
-       the PVD, do that.
-     */
-    if ( cdio_charset_to_utf8(p_iso->svd.system_id, ISO_MAX_SYSTEM_ID,
-                              p_psz_system_id, "UCS-2BE") )
-      return true;
-  }
-#endif /*HAVE_JOLIET*/
-  *p_psz_system_id = iso9660_get_system_id( &(p_iso->pvd) );
-  return *p_psz_system_id != NULL && strlen(*p_psz_system_id);
+  return get_member_id(p_iso, p_psz_system_id,
+                       (char*)p_iso->pvd.system_id,
+                       (char*)p_iso->svd.system_id,
+                       ISO_MAX_SYSTEM_ID);
 }
 
-
 /*!
    Return a string containing the PVD's publisher id with trailing
    blanks removed.
@@ -411,28 +414,12 @@ bool iso9660_ifs_get_system_id(iso9660_t *p_iso,
 bool iso9660_ifs_get_volume_id(iso9660_t *p_iso,
 			       /*out*/ cdio_utf8_t **p_psz_volume_id)
 {
-  if (!p_iso) {
-    *p_psz_volume_id = NULL;
-    return false;
-  }
-
-#ifdef HAVE_JOLIET  
-  if (p_iso->i_joliet_level) {
-    /* TODO: check that we haven't reached the maximum size.
-       If we have, perhaps we've truncated and if we can get 
-       longer results *and* have the same character using
-       the PVD, do that.
-     */
-    if ( cdio_charset_to_utf8(p_iso->svd.volume_id, ISO_MAX_VOLUME_ID,
-                              p_psz_volume_id, "UCS-2BE") )
-      return true;
-  }
-#endif /* HAVE_JOLIET */
-  *p_psz_volume_id = iso9660_get_volume_id( &(p_iso->pvd) );
-  return *p_psz_volume_id != NULL && strlen(*p_psz_volume_id);
+  return get_member_id(p_iso, p_psz_volume_id,
+                       (char*)p_iso->pvd.volume_id,
+                       (char*)p_iso->svd.volume_id,
+                       ISO_MAX_VOLUME_ID);
 }
 
-
 /*!
    Return a string containing the PVD's publisher id with trailing
    blanks removed.
@@ -440,27 +427,10 @@ bool iso9660_ifs_get_volume_id(iso9660_t *p_iso,
 bool iso9660_ifs_get_volumeset_id(iso9660_t *p_iso,
 				  /*out*/ cdio_utf8_t **p_psz_volumeset_id)
 {
-  if (!p_iso) {
-    *p_psz_volumeset_id = NULL;
-    return false;
-  }
-
-#ifdef HAVE_JOLIET  
-  if (p_iso->i_joliet_level) {
-    /* TODO: check that we haven't reached the maximum size.
-       If we have, perhaps we've truncated and if we can get 
-       longer results *and* have the same character using
-       the PVD, do that.
-     */
-    if ( cdio_charset_to_utf8(p_iso->svd.volume_set_id, 
-                              ISO_MAX_VOLUMESET_ID, 
-                              p_psz_volumeset_id,
-                              "UCS-2BE") )
-      return true;
-  }
-#endif /*HAVE_JOLIET*/
-  *p_psz_volumeset_id = iso9660_get_volumeset_id( &(p_iso->pvd) );
-  return *p_psz_volumeset_id != NULL && strlen(*p_psz_volumeset_id);
+  return get_member_id(p_iso, p_psz_volumeset_id,
+                       (char*)p_iso->pvd.volume_set_id,
+                       (char*)p_iso->svd.volume_set_id,
+                       ISO_MAX_VOLUMESET_ID);
 }
 
 
@@ -500,19 +470,25 @@ bool
 iso9660_ifs_read_superblock (iso9660_t *p_iso, 
 			     iso_extension_mask_t iso_extension_mask)
 {
-  iso9660_svd_t *p_svd;  /* Secondary volume descriptor. */
-  
+  iso9660_svd_t p_svd;  /* Secondary volume descriptor. */
+  int i;
+
   if (!p_iso || !iso9660_ifs_read_pvd(p_iso, &(p_iso->pvd)))
     return false;
 
-  p_svd = &(p_iso->svd);
   p_iso->i_joliet_level = 0;
 
-  if (0 != iso9660_iso_seek_read (p_iso, p_svd, ISO_PVD_SECTOR+1, 1)) {
-    if ( ISO_VD_SUPPLEMENTARY == from_711(p_svd->type) ) {
-      if (p_svd->escape_sequences[0] == 0x25 
-	  && p_svd->escape_sequences[1] == 0x2f) {
-	switch (p_svd->escape_sequences[2]) {
+  /* There may be multiple Secondary Volume Descriptors (eg. El Torito + Joliet) */
+  for (i=1; (0 != iso9660_iso_seek_read (p_iso, &p_svd, ISO_PVD_SECTOR+i, 1)); i++) {
+    if (ISO_VD_END == from_711(p_svd.type) ) /* Last SVD */
+      break;
+    if ( ISO_VD_SUPPLEMENTARY == from_711(p_svd.type) ) {
+      /* We're only interested in Joliet => make sure the SVD isn't overwritten */
+      if (p_iso->i_joliet_level == 0)
+        memcpy(&(p_iso->svd), &p_svd, sizeof(iso9660_svd_t));
+      if (p_svd.escape_sequences[0] == 0x25 
+	  && p_svd.escape_sequences[1] == 0x2f) {
+	switch (p_svd.escape_sequences[2]) {
 	case 0x40:
 	  if (iso_extension_mask & ISO_EXTENSION_JOLIET_LEVEL1) 
 	    p_iso->i_joliet_level = 1;
@@ -709,7 +685,7 @@ iso9660_seek_read_framesize (const iso9660_t *p_iso, void *ptr,
 			     uint16_t i_framesize)
 {
   long int ret;
-  long int i_byte_offset;
+  int64_t i_byte_offset;
   
   if (!p_iso) return 0;
   i_byte_offset = (start * p_iso->i_framesize) + p_iso->i_fuzzy_offset 
@@ -743,7 +719,7 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
 
   if (!dir_len) return NULL;
 
-  i_fname  = from_711(p_iso9660_dir->filename_len);
+  i_fname  = from_711(p_iso9660_dir->filename.len);
 
   /* .. string in statbuf is one longer than in p_iso9660_dir's listing '\1' */
   stat_len      = sizeof(iso9660_stat_t)+i_fname+2;
@@ -779,7 +755,7 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
 	  calloc(1, sizeof(iso9660_stat_t)+i_rr_fname+2);
         if (!p_stat_new)
           {
-          cdio_warn("Couldn't calloc(1, %d)", sizeof(iso9660_stat_t)+i_rr_fname+2);
+          cdio_warn("Couldn't calloc(1, %d)", (int)(sizeof(iso9660_stat_t)+i_rr_fname+2));
           return NULL;
           }
 	memcpy(p_stat_new, p_stat, stat_len);
@@ -788,26 +764,27 @@ _iso9660_dir_to_statbuf (iso9660_dir_t *p_iso9660_dir, bool_3way_t b_xa,
       }
       strncpy(p_stat->filename, rr_fname, i_rr_fname+1);
     } else {
-      if ('\0' == p_iso9660_dir->filename[0] && 1 == i_fname)
+      if ('\0' == p_iso9660_dir->filename.str[1] && 1 == i_fname)
 	strncpy (p_stat->filename, ".", sizeof("."));
-      else if ('\1' == p_iso9660_dir->filename[0] && 1 == i_fname)
+      else if ('\1' == p_iso9660_dir->filename.str[1] && 1 == i_fname)
 	strncpy (p_stat->filename, "..", sizeof(".."));
 #ifdef HAVE_JOLIET
       else if (i_joliet_level) {
 	int i_inlen = i_fname;
 	cdio_utf8_t *p_psz_out = NULL;
-	if (cdio_charset_to_utf8(p_iso9660_dir->filename, i_inlen,
+	if (cdio_charset_to_utf8(&p_iso9660_dir->filename.str[1], i_inlen,
                              &p_psz_out, "UCS-2BE")) {
           strncpy(p_stat->filename, p_psz_out, i_fname);
           free(p_psz_out);
         }
         else {
+          free(p_stat);
           return NULL;
         }
       }
 #endif /*HAVE_JOLIET*/
       else {
-	strncpy (p_stat->filename, p_iso9660_dir->filename, i_fname);
+	strncpy (p_stat->filename, &p_iso9660_dir->filename.str[1], i_fname);
       }
     }
   }
@@ -881,12 +858,12 @@ iso9660_dir_to_name (const iso9660_dir_t *iso9660_dir)
 
   /* (iso9660_dir->file_flags & ISO_DIRECTORY) */
   
-  if (iso9660_dir->filename[0] == '\0')
+  if (iso9660_dir->filename.str[1] == '\0')
     return strdup(".");
-  else if (iso9660_dir->filename[0] == '\1')
+  else if (iso9660_dir->filename.str[1] == '\1')
     return strdup("..");
   else {
-    return strdup(iso9660_dir->filename);
+    return strdup(&iso9660_dir->filename.str[1]);
   }
 }
 
@@ -985,13 +962,6 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
 
   cdio_assert (_root->type == _STAT_DIR);
 
-  if (_root->size != ISO_BLOCKSIZE * _root->secsize)
-    {
-      cdio_warn ("bad size for ISO9660 directory (%ud) should be (%lu)!",
-		 (unsigned) _root->size, 
-		 (unsigned long int) ISO_BLOCKSIZE * _root->secsize);
-    }
-  
   _dirbuf = calloc(1, _root->secsize * ISO_BLOCKSIZE);
   if (!_dirbuf)
     {
@@ -1006,7 +976,7 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
   while (offset < (_root->secsize * ISO_BLOCKSIZE))
     {
       iso9660_dir_t *p_iso9660_dir = (void *) &_dirbuf[offset];
-      iso9660_stat_t *p_stat;
+      iso9660_stat_t *p_iso9660_stat;
       int cmp;
 
       if (!iso9660_get_dir_len(p_iso9660_dir))
@@ -1015,26 +985,26 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
 	  continue;
 	}
       
-      p_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, dunno, 
+      p_iso9660_stat = _iso9660_dir_to_statbuf (p_iso9660_dir, dunno, 
 					p_env->i_joliet_level);
 
-      cmp = strcmp(splitpath[0], p_stat->filename);
+      cmp = strcmp(splitpath[0], p_iso9660_stat->filename);
 
       if ( 0 != cmp && 0 == p_env->i_joliet_level 
-	   && yep != p_stat->rr.b3_rock ) {
+	   && yep != p_iso9660_stat->rr.b3_rock ) {
 	char *trans_fname = NULL;
-	unsigned int i_trans_fname=strlen(p_stat->filename);
-	int trans_len;
+	unsigned int i_trans_fname=strlen(p_iso9660_stat->filename);
 	
 	if (i_trans_fname) {
 	  trans_fname = calloc(1, i_trans_fname+1);
 	  if (!trans_fname) {
 	    cdio_warn("can't allocate %lu bytes", 
-		      (long unsigned int) strlen(p_stat->filename));
+		      (long unsigned int) strlen(p_iso9660_stat->filename));
+	    free(p_iso9660_stat);
 	    return NULL;
 	  }
-	  trans_len = iso9660_name_translate_ext(p_stat->filename, trans_fname,
-						 p_env->i_joliet_level);
+	  iso9660_name_translate_ext(p_iso9660_stat->filename, trans_fname,
+				     p_env->i_joliet_level);
 	  cmp = strcmp(splitpath[0], trans_fname);
 	  free(trans_fname);
 	}
@@ -1042,15 +1012,15 @@ _fs_stat_traverse (const CdIo_t *p_cdio, const iso9660_stat_t *_root,
       
       if (!cmp) {
 	iso9660_stat_t *ret_stat 
-	  = _fs_stat_traverse (p_cdio, p_stat, &splitpath[1]);
-	free(p_stat->rr.psz_symlink);
-	free(p_stat);
+	  = _fs_stat_traverse (p_cdio, p_iso9660_stat, &splitpath[1]);
+	free(p_iso9660_stat->rr.psz_symlink);
+	free(p_iso9660_stat);
 	free (_dirbuf);
 	return ret_stat;
       }
 
-      free(p_stat->rr.psz_symlink);
-      free(p_stat);
+      free(p_iso9660_stat->rr.psz_symlink);
+      free(p_iso9660_stat);
 	  
       offset += iso9660_get_dir_len(p_iso9660_dir);
     }
@@ -1092,13 +1062,6 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 
   cdio_assert (_root->type == _STAT_DIR);
 
-  if (_root->size != ISO_BLOCKSIZE * _root->secsize)
-    {
-      cdio_warn ("bad size for ISO9660 directory (%ud) should be (%lu)!",
-		 (unsigned) _root->size, 
-		 (unsigned long int) ISO_BLOCKSIZE * _root->secsize);
-    }
-  
   _dirbuf = calloc(1, _root->secsize * ISO_BLOCKSIZE);
   if (!_dirbuf)
     {
@@ -1130,17 +1093,17 @@ _fs_iso_stat_traverse (iso9660_t *p_iso, const iso9660_stat_t *_root,
 	   && yep != p_stat->rr.b3_rock ) {
 	char *trans_fname = NULL;
 	unsigned int i_trans_fname=strlen(p_stat->filename);
-	int trans_len;
 	
 	if (i_trans_fname) {
 	  trans_fname = calloc(1, i_trans_fname+1);
 	  if (!trans_fname) {
 	    cdio_warn("can't allocate %lu bytes", 
 		      (long unsigned int) strlen(p_stat->filename));
+	    free(p_stat);
 	    return NULL;
 	  }
-	  trans_len = iso9660_name_translate_ext(p_stat->filename, trans_fname, 
-						 p_iso->i_joliet_level);
+	  iso9660_name_translate_ext(p_stat->filename, trans_fname, 
+				     p_iso->i_joliet_level);
 	  cmp = strcmp(splitpath[0], trans_fname);
 	  free(trans_fname);
 	}
@@ -1305,13 +1268,6 @@ iso9660_fs_readdir (CdIo_t *p_cdio, const char psz_path[], bool b_mode2)
     uint8_t *_dirbuf = NULL;
     CdioList_t *retval = _cdio_list_new ();
 
-    if (p_stat->size != ISO_BLOCKSIZE * p_stat->secsize)
-      {
-	cdio_warn ("bad size for ISO9660 directory (%ud) should be (%lu)!",
-		   (unsigned) p_stat->size, 
-		   (unsigned long int) ISO_BLOCKSIZE * p_stat->secsize);
-      }
-
     _dirbuf = calloc(1, p_stat->secsize * ISO_BLOCKSIZE);
     if (!_dirbuf)
       {
@@ -1376,13 +1332,6 @@ iso9660_ifs_readdir (iso9660_t *p_iso, const char psz_path[])
     uint8_t *_dirbuf = NULL;
     CdioList_t *retval = _cdio_list_new ();
 
-    if (p_stat->size != ISO_BLOCKSIZE * p_stat->secsize)
-      {
-	cdio_warn ("bad size for ISO9660 directory (%ud) should be (%lu)!",
-		   (unsigned int) p_stat->size, 
-		   (unsigned long int) ISO_BLOCKSIZE * p_stat->secsize);
-      }
-
     _dirbuf = calloc(1, p_stat->secsize * ISO_BLOCKSIZE);
     if (!_dirbuf)
       {
@@ -1391,7 +1340,11 @@ iso9660_ifs_readdir (iso9660_t *p_iso, const char psz_path[])
       }
 
     ret = iso9660_iso_seek_read (p_iso, _dirbuf, p_stat->lsn, p_stat->secsize);
-    if (ret != ISO_BLOCKSIZE*p_stat->secsize) return NULL;
+    if (ret != ISO_BLOCKSIZE*p_stat->secsize) 
+	  {
+	    free (_dirbuf);
+	    return NULL;
+	  }
     
     while (offset < (p_stat->secsize * ISO_BLOCKSIZE))
       {
@@ -1421,6 +1374,7 @@ iso9660_ifs_readdir (iso9660_t *p_iso, const char psz_path[])
       return NULL;
     }
 
+    free (p_stat->rr.psz_symlink);
     free (p_stat);
     return retval;
   }
@@ -1446,7 +1400,7 @@ find_lsn_recurse (void *p_image, iso9660_readdir_t iso9660_readdir,
     {
       iso9660_stat_t *statbuf = _cdio_list_node_data (entnode);
       const char *psz_filename  = (char *) statbuf->filename;
-      const unsigned int len = strlen(psz_path) + strlen(psz_filename)+2;
+      unsigned int len = strlen(psz_path) + strlen(psz_filename)+2;
       
       if (*ppsz_full_filename != NULL) free(*ppsz_full_filename);
       *ppsz_full_filename = calloc(1, len);
@@ -1459,14 +1413,14 @@ find_lsn_recurse (void *p_image, iso9660_readdir_t iso9660_readdir,
       }
 
       if (statbuf->lsn == lsn) {
-	unsigned int len=sizeof(iso9660_stat_t)+strlen(statbuf->filename)+1;
-	iso9660_stat_t *ret_stat = calloc(1, len);
+	const unsigned int len2 = sizeof(iso9660_stat_t)+strlen(statbuf->filename)+1;
+	iso9660_stat_t *ret_stat = calloc(1, len2);
 	if (!ret_stat)
 	  {
-          cdio_warn("Couldn't calloc(1, %d)", len);
+          cdio_warn("Couldn't calloc(1, %d)", len2);
           return NULL;
 	  }
-	memcpy(ret_stat, statbuf, len);
+	memcpy(ret_stat, statbuf, len2);
         _cdio_list_free (entlist, true);
         _cdio_list_free (dirlist, true);
         return ret_stat;
