@@ -1,7 +1,6 @@
 /*
-  $Id: cd-read.c,v 1.33 2008/06/19 15:44:13 flameeyes Exp $
-
-  Copyright (C) 2003, 2004, 2005, 2006, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2003, 2004, 2005, 2006, 2008, 2011 Rocky Bernstein 
+  <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -434,7 +433,7 @@ parse_options (int argc, char *argv[])
     /* We were given an end lsn. */
     if (opts.end_lsn < opts.start_lsn) {
       report( stderr, 
-	      "%s: end LSN (%lu) needs to be less than start LSN (%lu)\n",
+	      "%s: end LSN (%lu) needs to be grater than start LSN (%lu)\n",
 	      program_name, (unsigned long) opts.start_lsn, 
 	      (unsigned long) opts.end_lsn );
       free(program_name);
@@ -546,10 +545,13 @@ main(int argc, char *argv[])
 	  break;
 	case READ_M1F2:
 	  blocklen=M2RAW_SECTOR_SIZE;
+	  break;
 	case READ_M2F1:
 	  blocklen=CDIO_CD_FRAMESIZE;
+	  break;
 	case READ_M2F2:
 	  blocklen=M2F2_SECTOR_SIZE;
+	  break;
 	default: ;
 	}
       }
@@ -586,9 +588,11 @@ main(int argc, char *argv[])
     
     if (opts.hexdump)
       hexdump(output_stream, buffer, blocklen, opts.just_hex);
-    else if (opts.output_file)
-      write(output_fd, buffer, blocklen);
-    else {
+    else if (opts.output_file) {
+      ssize_t bytes_ret;
+      bytes_ret = write(output_fd, buffer, blocklen);
+      (void)bytes_ret; /* Silence unused warnings */
+    } else {
       unsigned int i;
       for (i=0; i<blocklen; i++) printf("%c", buffer[i]);
     }
