@@ -17,11 +17,12 @@
 
 /* A program to using the MMC interface to list CD and drive features
    from the MMC GET_CONFIGURATION command . */
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-# define __CDIO_CONFIG_H__ 1
-#endif
+
+#include "util.h"
+
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
+#endif
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -34,7 +35,6 @@
 #include <cdio/cdio.h>
 #include <cdio/mmc.h>
 #include <cdio/mmc_cmds.h>
-#include "util.h"
 #include "getopt.h"
 
 static void
