@@ -1,7 +1,6 @@
 /*
-  $Id: util.c,v 1.54 2008/04/14 17:30:27 karl Exp $
-
-  Copyright (C) 2003, 2004, 2005, 2007, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009, 2010, 2012
+  Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,6 +26,16 @@
 # include <sys/stat.h>
 #endif
 
+/* MSYS 1.0.10 with MinGW 3.4.2 (and perhaps others) don't have
+   S_ISSOCK() or S_ISLNK() macros, so we'll roll our own. */
+#if !defined(HAVE_S_ISSOCK) && !defined(S_ISSOCK)
+#define S_ISSOCK(st_mode) ((((st_mode)) & 0170000) == (0140000))
+#endif
+
+#if !defined(HAVE_S_ISLNK) && !defined(S_ISLNK)
+#define S_ISLNK(st_mode) ((((st_mode)) & 0170000) == (0010000))
+#endif
+
 cdio_log_handler_t gl_default_cdio_log_handler = NULL;
 char *source_name = NULL;
 char *program_name;
@@ -45,23 +54,25 @@ print_version (char *program_name, const char *version,
 	       int no_header, bool version_only)
 {
   
-  driver_id_t driver_id;
-
-  if (no_header == 0)
+  if (no_header == 0) {
     report( stdout,  
-	    "%s version %s\nCopyright (c) 2003, 2004, 2005, 2007, 2008 R. Bernstein\n",
+	    "%s version %s\n"
+	    "Copyright (c) 2003, 2004, 2005, 2007, 2008, 2011, 2012 "
+	    "R. Bernstein\n",
 	    program_name, version);
-  report( stdout,  
-	  _("This is free software; see the source for copying conditions.\n\
+    report( stdout,  
+            _("This is free software; see the source for copying conditions.\n\
 There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n\
 PARTICULAR PURPOSE.\n\
 "));
+  }
 
   if (version_only) {
     char *default_device;
-    for (driver_id=DRIVER_UNKNOWN+1; driver_id<=CDIO_MAX_DRIVER; driver_id++) {
-      if (cdio_have_driver(driver_id)) {
-	report( stdout, "Have driver: %s\n", cdio_driver_describe(driver_id));
+    const driver_id_t *driver_id_p;
+    for (driver_id_p=cdio_drivers; *driver_id_p!=DRIVER_UNKNOWN; driver_id_p++) {
+      if (cdio_have_driver(*driver_id_p)) {
+	report( stdout, "Have driver: %s\n", cdio_driver_describe(*driver_id_p));
       }
     }
     default_device=cdio_get_default_device(NULL);
@@ -160,7 +171,7 @@ open_input(const char *psz_source, source_image_t source_image,
 char *
 fillout_device_name(const char *device_name) 
 {
-#if defined(HAVE_WIN32_CDROM)
+#if defined(HAVE_WIN32_CDROM) || defined(HAVE_OS2_CDROM)
   return strdup(device_name);
 #else
   unsigned int prefix_len = strlen(DEV_PREFIX);
