@@ -1,6 +1,4 @@
 /*
-  $Id: testdefault.c,v 1.8 2008/03/22 18:08:25 karl Exp $
-
   Copyright (C) 2003, 2004, 2005, 2008 Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
@@ -22,11 +20,9 @@
    and cdio_free_device_list()
 */
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#include "config.h"
+#define __CDIO_CONFIG_H__ 1
 #endif
-#include <cdio/cdio.h>
-#include <cdio/cd_types.h>
-#include <cdio/logging.h>
 
 #ifdef HAVE_STDIO_H
 #include <stdio.h>
@@ -37,7 +33,13 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+
+#include <cdio/cdio.h>
+#include <cdio/cd_types.h>
+#include <cdio/logging.h>
 
 static void 
 log_handler (cdio_log_level_t level, const char message[])
@@ -80,6 +82,13 @@ main(int argc, const char *argv[])
   
   cdio_log_set_handler (log_handler);
 
+  if (cdio_have_driver(-1) != false) 
+    {
+      printf("Bogus driver number -1 should be regexted\n");
+      return 5;
+    }
+  
+
   if (! (cdio_have_driver(DRIVER_NRG) && cdio_have_driver(DRIVER_BINCUE)) )  {
     printf("You don't have enough drivers for this test\n");
     exit(77);
@@ -189,12 +198,8 @@ main(int argc, const char *argv[])
   }
     
   cdio_free_device_list(nrg_images);
-  free(nrg_images);
   cdio_free_device_list(bincue_images);
-  free(bincue_images);
   cdio_free_device_list(imgs);
-  free(imgs);
-
   return 0;
   
 }
