@@ -1,35 +1,24 @@
-/* $Id: testisocd.c,v 1.5 2007/08/12 12:41:10 rocky Exp $
-
-  Copyright (C) 2003, 2004, 2005, 2006, 2007 Rocky Bernstein 
+/* 
+  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2011 Rocky Bernstein 
   <rocky@gnu.org>
   
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
+  the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
-  
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
-  
+
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* Tests reading ISO 9660 info from a CD.  */
-
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
 #include "portable.h"
 
-#include <cdio/cdio.h>
-#include <cdio/iso9660.h>
-#include <cdio/cd_types.h>
-
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -48,9 +37,10 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
+
+#include <cdio/cdio.h>
+#include <cdio/iso9660.h>
+#include <cdio/cd_types.h>
 
 #define SKIP_TEST_RC 77
 
@@ -75,7 +65,7 @@ main(int argc, const char *argv[])
     /* Don't need a list of CD's with CD-DA's any more. */
     cdio_free_device_list(ppsz_cd_drives);
   } else {
-    printf("Unable find or access a CD-ROM drive with an ISO-9660 "
+    printf("-- Unable find or access a CD-ROM drive with an ISO-9660 "
 	   "filesystem.\n");
     exit(SKIP_TEST_RC);
   }
