@@ -1,6 +1,5 @@
-/* $Id: testisocd2.c.in,v 1.2 2008/03/22 18:08:25 karl Exp $
-
-  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008 Rocky Bernstein 
+/* 
+  Copyright (C) 2003-2008, 2012 Rocky Bernstein 
   <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
@@ -18,17 +17,12 @@
 */
 
 /* Tests reading ISO 9660 info from an ISO 9660 image.  */
+#include "portable.h"
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#include "portable.h"
-
-#include <cdio/cdio.h>
-#include <cdio/iso9660.h>
-#include <cdio/cd_types.h>
-
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -47,9 +41,11 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
+
+#include <cdio/cdio.h>
+#include <cdio/iso9660.h>
+#include <cdio/cd_types.h>
+#include "filemode.h"
 
 /* Set up a CD-DA image to test on which is in the libcdio distribution. */
 #define ISO9660_IMAGE_PATH "/src/external-vcs/libcdio/test/data/"
